package ship;

import java.util.EnumSet;
import java.util.LinkedList;
import java.util.List;

import VASSAL.counters.GamePiece;

import spaceAlert.GamePieceCreator;
import threats.ThreatZones;

public class DamageGenerator {
	protected List<ShipDamage> workingSystemsInZoneRed = null;
	protected List<ShipDamage> workingSystemsInZoneWhite = null;
	protected List<ShipDamage> workingSystemsInZoneBlue = null;
	
	public int getDamageCountIn(ThreatZones zone) {
		switch (zone) {
		case BLUE:
			return 6 - workingSystemsInZoneBlue.size();
		case RED:
			return 6 - workingSystemsInZoneRed.size();
		case WHITE:
			return 6 - workingSystemsInZoneWhite.size();
		default:
			return 0;
		}
	}
	public GamePiece getDamagePiece(ShipDamage damage) {
		if (damage == ShipDamage.DESTROYED) {
			return null;
		} else {
			GamePieceCreator gamePieceCreator = GamePieceCreator.getInstance();
			return gamePieceCreator.get(damage.toString());
		}
	}

	public ShipDamage getDamage(ThreatZones zone) {	
		switch (zone) {
		case BLUE:
			return getDamageFrom(workingSystemsInZoneBlue);
		case RED:
			return getDamageFrom(workingSystemsInZoneRed);
		case WHITE:
			return getDamageFrom(workingSystemsInZoneWhite);
		default:
			return ShipDamage.DESTROYED;
		}
	}
	
	protected ShipDamage getDamageFrom(List<ShipDamage> workingSystems) {
		if (workingSystems.size() < 1) {
			return ShipDamage.DESTROYED;
		} else {
			int randomSystem = (int)(Math.random() * (workingSystems.size() - 1));
			return workingSystems.remove(randomSystem);
		}
	}

	DamageGenerator(List<ShipDamage> damagedSystems) {
		workingSystemsInZoneRed = new LinkedList<ShipDamage>();
		workingSystemsInZoneRed.addAll(EnumSet.range(ShipDamage.RED_LIGHT_LASER_CANNON, ShipDamage.RED_ELEVATOR));
		workingSystemsInZoneRed.removeAll(damagedSystems);
		workingSystemsInZoneWhite = new LinkedList<ShipDamage>();
		workingSystemsInZoneWhite.addAll(EnumSet.range(ShipDamage.WHITE_PULSE_CANNON, ShipDamage.WHITE_ELEVATOR));
		workingSystemsInZoneWhite.removeAll(damagedSystems);
		workingSystemsInZoneBlue = new LinkedList<ShipDamage>();
		workingSystemsInZoneBlue.addAll(EnumSet.range(ShipDamage.BLUE_LIGHT_LASER_CANNON, ShipDamage.BLUE_ELEVATOR));
		workingSystemsInZoneBlue.removeAll(damagedSystems);
	}
}
