package ship;

import VASSAL.build.module.map.boardPicker.board.Region;
import VASSAL.counters.FreeRotator;
import VASSAL.counters.GamePiece;
import spaceAlert.CardAction;
import spaceAlert.GamePieceCreator;
import spaceAlert.Log;
import spaceAlert.Nameable;
import spaceAlert.VassalHelper;
import threats.InternalAttachedThreatBase;

public class Fighters extends WeaponBase {
	protected GamePiece fighters;
	protected Player mannedBy = null;
	
	private Region landed, started;
	
	protected Optics optics;
	
	public boolean man(Player player) {
		if ((mannedBy == null)
		&& (player.hasBattlebots())) {
			Log.log(player.getName() + " takes of in the interceptors.");
			mannedBy = player;
			player.moveTo(ShipZones.FIGHTERS);
			map.placeOrMerge(fighters, started.getOrigin());
			FreeRotator freeRotator = VassalHelper.getFreeRotator(fighters);
			if (freeRotator != null) {
				freeRotator.setAngle(-90);
			}
			fire(player, false);
			return true;
		} else {
			Log.log(player.getName() + " tries taking of in the interceptors but they're already in space.");
			return false;
		}
	}
	
	public void unMan() {
		if (mannedBy != null) {
			Log.log(mannedBy.getName() + " returns to ship.");
			
			mannedBy.moveTo(ShipZones.RED_UPPER);
			mannedBy = null;
			map.placeOrMerge(fighters, landed.getOrigin());
			FreeRotator freeRotator = VassalHelper.getFreeRotator(fighters);
			if (freeRotator != null) {
				freeRotator.setAngle(0);
			}
		}
	}
	
	public boolean isManned() {
		return mannedBy != null;
	}
	
	public Player getManned() {
		return mannedBy;
	}
	
	protected boolean multipleTargets;
	public void setMultipleTargets(boolean multipleTargets) {
		this.multipleTargets = multipleTargets;
	}
	
	public void doPlayerAction(Player player, CardAction action) {
		switch (action) {
		case ACTIVATE_BOTS:
			fire(player, false);
			break;
		case HEROIC_BOT:
			fire(player, true);
			break;
		case NOTHING:
			unMan();
			break;
		case PRESS_C:
		case MOVE_TO_BLUE:
		case MOVE_TO_RED:
		case PRESS_A:
		case PRESS_B:
		case SWITCH_DECK:
		case HEROIC_A:
		case HEROIC_B:
		case HEROIC_MOVE_TO_BLUE_LOWER:
		case HEROIC_MOVE_TO_BLUE_UPPER:
		case HEROIC_MOVE_TO_RED_LOWER:
		case HEROIC_MOVE_TO_RED_UPPER:
		case HEROIC_MOVE_TO_WHITE_LOWER:
		case HEROIC_MOVE_TO_WHITE_UPPER:
		default:
			unMan();
			player.delayNow();		
		}
	}

	public Fighters(Optics optics) {
		super(optics);
		landed = map.findRegion("Red: Fighters");
		started = map.findRegion("Space: Fighters");
		
		this.optics = optics;
		
		GamePieceCreator gamePieceCreator = GamePieceCreator.getInstance();
		fighters = gamePieceCreator.get("Fighters");
		map.addPiece(fighters);
		map.placeAt(fighters, landed.getOrigin());
	}
	
	@Override
	public WeaponRanges getRange() {
		return WeaponRanges.Range1;
	}

	@Override
	public int getDamage() {
		if (multipleTargets) {
			if (firesHeroic) {
				return 2;
			} else {
				return 1;
			}
		} else {
			if (firesHeroic) {
				return 4;
			} else {
				return 3;
			}
		}
	}

	@Override
	public void fire(Nameable player, boolean isHeroic) {
		InternalAttachedThreatBase threat = getBlocker();
		if (threat != null) {
			threat.damage(player, this, isHeroic);
		} else {
			optics.doVisualTargeting();
			Log.log(player.getName() + " attacks with the interceptors.");
			fired = true;
			firesHeroic = isHeroic;
		}
	}
	
	@Override
	public String getName() {
		return "interceptors";
	}

}
