package ship;

import spaceAlert.CardAction;
import spaceAlert.Log;
import spaceAlert.Space;
import threats.ThreatBase;
import yellowThreats.I2_01;
import yellowThreats.I2_102;

public class Movement {
	private static Space space;
	private static Ship ship;
	public static void initialize(Space space, Ship ship) {
		Movement.space = space;
		Movement.ship = ship;
	}
	
	static I2_102 ninja = null;
	public static void infect(I2_102 ninja) {
		Movement.ninja = ninja;
	}
	public static void unInfect() {
		ninja = null;
	}
	
	public static void moveTo(Moveable object, CardAction action) {
		ShipZones from = object.getLocation();
		ShipZones target;
		
		switch (action) {
		case MOVE_TO_RED:
			target = moveToRed(from);
			break;
		case MOVE_TO_BLUE:
			target = moveToBlue(from);
			break;
		case SWITCH_DECK:
			target = switchDeck(from);
			break;
		default:
			target = from;
		}
		if (ship.isMovementAllowed(from, target)) {
			Log.log(object.getName() + " moves to " + target.getName() + ".");
			object.moveTo(target);
			
			if (object instanceof Player) {
				Player player = (Player)object;
				//checking if delayed due to elevator
				checkElevatorUsage(action, target, player);
				checkDelaysDueInternalThreats(player, from, target);
			}
		} else {
			Log.log(object.getName() + " tries to move to " + target.getName() + " but failed.");
		}
	}
	
	public static void heroicMoveTo(Player player, ShipZones target) {
		ShipZones from = player.getLocation();
		ShipZones currentZone = from;
		ShipZones zoneOnTheWay = null;
		do {
			zoneOnTheWay = findWayToTarget(player, from, currentZone, target);
			
			if (!ship.isMovementAllowed(currentZone, zoneOnTheWay)) {
				if (currentZone.isUpper() != target.isUpper()) {
					currentZone = switchDeck(currentZone);
				}
				Log.log(player.getName() + "tries moves heroic to " + target.getName() + " but failed and ends in " + currentZone.getName() + ".");
				player.moveTo(currentZone);
				if (currentZone != from) {
					checkDelaysDueInternalThreats(player, from, currentZone);
				}
				return;
			} else {
				if (zoneOnTheWay == target) {
					Log.log(player.getName() + " moves heroic to " + target.getName() + ".");
					player.moveTo(target);
					checkDelaysDueInternalThreats(player, from, target);
					return;
				}	
			}
			currentZone = zoneOnTheWay;
		} while (zoneOnTheWay != null);
	}
	
	private static ShipZones findWayToTarget(Player player, ShipZones from, ShipZones currentZone, ShipZones target) {
		ShipZones zoneOnTheWay = null;
		ShipZones neighbors[] = currentZone.getNeighbour();
		for (ShipZones neighbor: neighbors) {
			
			if (neighbor == target) { //target is neighbor
				return neighbor;
			}  else if (neighbor.getThreatZone() != from.getThreatZone()) { //neighbor is in another zone
				zoneOnTheWay = neighbor;
				if (neighbor.getThreatZone() == target.getThreatZone()) {
					return neighbor;
				}
			}
		}
		return zoneOnTheWay;
	}
	
	private static void checkDelaysDueInternalThreats(Player player,
			ShipZones from, ShipZones target) {
		//checking for ninja
		if (ninja != null) {
			if ((target == ShipZones.BLUE_UPPER)
			|| (target == ShipZones.WHITE_LOWER)
			|| (from == ShipZones.BLUE_UPPER)
			|| (from == ShipZones.WHITE_LOWER)) {
				ninja.infect(player);
			}					
		}
		//checking for slime
		for (ThreatBase threat: space.getThreats()) {
			if (threat instanceof I2_01) {
				if (((I2_01) threat).getLocation() == target) {
					Log.log(threat.getName() + " delays " + player.getName());
					player.delay();
				}
			}
		}
	}
	private static void checkElevatorUsage(CardAction action,
			ShipZones target, Player player) {
		if (action == CardAction.SWITCH_DECK) {
			switch (target) {
			case BLUE_LOWER:
			case BLUE_UPPER:
				if (!ship.isBlueElevatorAvailable()) {
					player.delay();
				}
				ship.setBlueElevatorAvailable(false);
				break;
			case WHITE_LOWER:
			case WHITE_UPPER:
				if (!ship.isWhiteElevatorAvailable()) {
					player.delay();
				}
				ship.setWhiteElevatorAvailable(false);
				break;
			case RED_LOWER:
			case RED_UPPER:
				if (!ship.isRedElevatorAvailable()) {
					player.delay();
				}
				ship.setRedElevatorAvailable(false);
				break;
			default:
				break;
			}
		}
	}
	
	private static ShipZones moveToRed(ShipZones location) {
		switch(location) {
		case BLUE_LOWER:
			return ShipZones.WHITE_LOWER;
		case BLUE_UPPER:
			return ShipZones.WHITE_UPPER;
		case RED_LOWER:
		case WHITE_LOWER:
			return ShipZones.RED_LOWER;
		case RED_UPPER:
		case WHITE_UPPER:
		default:
			return ShipZones.RED_UPPER;
		case FIGHTERS:
			return ShipZones.FIGHTERS;
		}
	}
	
	private static ShipZones moveToBlue(ShipZones location) {
		switch(location) {
		case BLUE_LOWER:
		case WHITE_LOWER:
			return ShipZones.BLUE_LOWER;
		case BLUE_UPPER:
		case WHITE_UPPER:
		default:
			return ShipZones.BLUE_UPPER;
		case RED_LOWER:
			return ShipZones.WHITE_LOWER;
		case RED_UPPER:
			return ShipZones.WHITE_UPPER;
		case FIGHTERS:
			return ShipZones.FIGHTERS;
		}
	}
	
	private static ShipZones switchDeck(ShipZones location) {
		switch(location) {
		case BLUE_LOWER:
			return ShipZones.BLUE_UPPER;
		case BLUE_UPPER:
			return ShipZones.BLUE_LOWER;
		case RED_LOWER:
			return ShipZones.RED_UPPER;
		case RED_UPPER:
			return ShipZones.RED_LOWER;
		case WHITE_LOWER:
			return ShipZones.WHITE_UPPER;
		case WHITE_UPPER:
		default:
			return ShipZones.WHITE_LOWER;
		case FIGHTERS:
			return ShipZones.FIGHTERS;
		}
	}
}
