package ship;

import spaceAlert.CardAction;
import spaceAlert.Log;
import spaceAlert.GamePieceCreator;
import spaceAlert.PlayersCards;
import spaceAlert.Space;
import VASSAL.build.module.Map;
import VASSAL.build.module.map.boardPicker.board.Region;
import VASSAL.command.Command;
import VASSAL.command.RemovePiece;
import VASSAL.counters.GamePiece;

public class Player implements Moveable {
	private PlayersCards cards;
	private GamePiece piece;
	private Battlebots battlebots = null;
	private Map map;
	private ShipZones location;
	public ShipZones getLocation() {
		return location;
	}
	
	private Region redUpper, redLower, whiteUpper, whiteLower, blueUpper, blueLower;
	
	private boolean unconscious = false;
	public boolean isUnconscious() {
		return unconscious;
	}
	public void knockOut() {
		unconscious = true;
		piece.setProperty("unconscious", 1);
	}
	
	private boolean poisoned = false;
	public void poison() {
		poisoned = true;
		piece.setProperty("poison", 1);
	}
	
	private void getPlayer() {
		GamePieceCreator gamePieceCreator = GamePieceCreator.getInstance();
		piece = gamePieceCreator.get("Player" + cards.getColor().toString());
		map.addPiece(piece);
		if (battlebots == null) {
			piece.setProperty("battlebots", 0);
		} else {
			if (battlebots.damaged) {
				piece.setProperty("battlebots", 2);
			} else {
				piece.setProperty("battlebots", 1);
			}
		}
		if (unconscious) {
			knockOut();
		}
		if (poisoned) {
			poison();
		}
	}
	
	private void deletePlayer() {
		Command c = new RemovePiece(piece);
		c.execute();
	}
	
	public void addBattlebots(Battlebots battlebots) {
		if (this.battlebots != null) {
			Log.log(getName() + " tries to activate a battlebot squad but is already followed by a squad.");
		} else {
			if (battlebots.isDestroyed()) {
				Log.log(getName() + " tries to activate a battlebot squad but that squad was destroyed.");
			} else {
				if (battlebots.follow(piece)) {
					this.battlebots = battlebots;
					Log.log(getName() + " activates a battlebot squad.");
				} else {
					Log.log(getName() + " tries to activate a battlebot squad but failed.");
				}
			}
		}
	}
	public boolean hasBattlebots() {
		return battlebots != null;
	}
	public Battlebots getBattlebots() {
		return battlebots;
	}
	
	public Player(PlayersCards playerCard, Map map){
		redUpper = map.findRegion(ShipZones.RED_UPPER.toString());
		redLower = map.findRegion(ShipZones.RED_LOWER.toString());
		whiteUpper = map.findRegion(ShipZones.WHITE_UPPER.toString());
		whiteLower = map.findRegion(ShipZones.WHITE_LOWER.toString());
		blueUpper = map.findRegion(ShipZones.BLUE_UPPER.toString());
		blueLower = map.findRegion(ShipZones.BLUE_LOWER.toString());
		
		cards = playerCard;
		this.map = map;
		getPlayer();
		map.placeOrMerge(piece, whiteUpper.getOrigin());
		location = ShipZones.WHITE_UPPER;
	}
	
	public void moveTo(ShipZones location) {
		if (piece == null) {
			getPlayer();
		}
		Region region = null;
		switch (location) {
		case BLUE_LOWER:
			region = blueLower;
			break;
		case BLUE_UPPER:
			region = blueUpper;
			break;
		case RED_LOWER:
			region = redLower;
			break;
		case RED_UPPER:
			region = redUpper;
			break;
		case WHITE_LOWER:
			region = whiteLower;
			break;
		case WHITE_UPPER:
			region = whiteUpper;
			break;
		case FIGHTERS:
		default:
			deletePlayer();
			break;
		}
		if (region != null) {
			map.placeOrMerge(piece, region.getOrigin());
		}
		this.location = location;
	}
	public String getName() {
		return cards.getColor().toString();
	}
	
	public void delay() {
		cards.delayAt(Space.getCurrentTime() + 1);
	}
	public void delayNow() {
		cards.delayAt(Space.getCurrentTime());
	}
	public void delayAt(int time) {
		cards.delayAt(time);
	}
	public CardAction getActionAt(int time) {
		return cards.getActionAt(time);
	}
}
