package ship;

import spaceAlert.Log;
import spaceAlert.Nameable;
import spaceAlert.ResetableBase;
import threats.InternalAttachedThreatBase;
import whiteThreats.I1_07;

public class RocketThrower extends WeaponBase {
	protected Rocket rocket[] = new Rocket[3];

	public RocketThrower(ResetableBase parent) {
		super(parent);
		
		rocket[0] = new Rocket(map, "Blue: Rocket 1");
		rocket[1] = new Rocket(map, "Blue: Rocket 2");
		rocket[2] = new Rocket(map, "Blue: Rocket 3");
	}
	
	@Override
	public void clear() {
		super.clear();
		
		for (int i = 0; i < 3; i++) {
			rocket[i].destroy();
		}
	}
	
	public void deleteRocket(Nameable player) {
		for (int i = 0; i < 3; i++) {
			if (!rocket[i].hasMoved()) {
				for (InternalAttachedThreatBase threat : blockedBy) {
					if ((threat instanceof I1_07) 
					&&threat.isAppeared()
					&& !threat.isDestroyed()) {
						threat.damage(player, this, false);
					}
				}
				rocket[i].destroy();
				break;
			}
		}
	}

	@Override
	public int getDamage() {
		for (int i = 0; i < 3; i++) {
			if ((rocket[i].hasMoved())
			&& (!rocket[i].isDestroyed())) {
				return 3;
			}
		}
		return 0;
	}

	@Override
	public void fire(Nameable player, boolean isHeroic) {
		InternalAttachedThreatBase threat = getBlocker();
		if (threat != null) {
			threat.damage(player, this, isHeroic);
		} else {
			if (!fired) {
				for (int i = 0; i < 3; i++) {
					if (!rocket[i].hasFired()) {
						rocket[i].fire();
						fired = true;
						Log.log(player.getName() + " launches a rocket.");
						return;
					}
				}
				Log.log(player.getName() + " tries to launch a rocket but none is available.");
			}
		}
	}
	
	@Override
	public boolean hasFired() {
		for (int i = 0; i < 3; i++) {
			if ((rocket[i].hasMoved())
			&& (!rocket[i].isDestroyed())) {
				return true;
			}
		}
		return false;
	}
	
	public void moveRockets() {
		for (int i = 0; i < 3; i++) {
			if ((rocket[i].hasFired())
			&& (!rocket[i].isDestroyed())) {
				rocket[i].move();
			}
		}
	}
	
	@Override
	public void reset() {
		super.reset();
		
		for (int i = 0; i < 3; i++) {
			if ((rocket[i].hasMoved())
			&& (!rocket[i].isDestroyed())) {
				rocket[i].destroy();
			}
		}
	}
	
	public int getRocketCount() {
		int count = 3;
		for (int i = 0; i < 3; i++) {
			if (rocket[i].hasMoved()) {
				count--;
			}
		}
		return count;
	}
	
	public String getName() {
		return "rocket thrower";
	}

}
