package ship;

import java.util.List;

import pieces.Energy;
import spaceAlert.Log;
import spaceAlert.Nameable;
import threats.InternalAttachedThreatBase;
import threats.ThreatZones;

public class SecondaryReactor extends ReactorBase {
	@Override
	protected int getMaximumEnergy() {
		if (damaged) {
			return 2;
		} else {
			return 3;
		}
	}
	protected PrimaryReactor primaryReactor;
	
	public void getEnergyFromPrimaryReactor(Nameable player, boolean isHeroic) {
		InternalAttachedThreatBase threat = getBlocker();
		if (threat != null) {
			threat.damage(player, this, isHeroic);
		} else {
			List<Energy> newEnergies = primaryReactor.getEnergy(getMaximumEnergy() - energies.size());
			for (Energy energy: newEnergies) {
				energy.moveTo(location);
			}
			if (isHeroic) {
				Energy energy = new Energy(map);
				energy.moveTo(location);
				energies.add(energy);
			}
			energies.addAll(newEnergies);
			Log.log(player.getName() + " transfers energy from " + primaryReactor.getName() + " to " + getName() + ".");
		}
	}
	
	public SecondaryReactor(PrimaryReactor primaryReactor, ThreatZones zone, boolean damaged) {
		super(primaryReactor, damaged);
		
		switch (zone) {
		case BLUE:
			name = "blue lateral reactor";
			this.location = map.findRegion("Blue: Reactor");
			break;
		default:
			name = "red lateral reactor";
			this.location = map.findRegion("Red: Reactor");
			break;
		}
		this.primaryReactor = primaryReactor;
		
		energies.add(new Energy(map));
		if (!damaged) {
			energies.add(new Energy(map));
		}
		for (Energy energy: energies) {
			energy.moveTo(location);
		}
	}
}
