package ship;

import java.util.LinkedList;
import java.util.List;

import VASSAL.build.module.map.boardPicker.board.Region;

import pieces.Energy;
import spaceAlert.Log;
import spaceAlert.Nameable;
import threats.InternalAttachedThreatBase;
import threats.ThreatZones;

public abstract class ShieldBase extends ShipComponentBase implements Damageable {
	protected Region location;
	protected String name;
	protected ReactorBase reactor;
	
	protected boolean damaged = false;
	public boolean isDamaged() {
		return damaged;
	}
	public void damage() {
		damaged = true;
		while (energies.size() > getMaximumEnergy()) {
			Energy energy = energies.remove(0);
			energy.delete();
		}
	}
	
	protected List<Energy> energies = new LinkedList<Energy>();
	protected abstract int getMaximumEnergy(); 
	public int getShieldStrength() {
		return energies.size();
	}
	
	public int recieveDamage(int damage) {
		if (damage <= getShieldStrength()) {
			Log.log(getName() + " shields the whole damage."); 
			while (damage > 0) {
				Energy energy = energies.remove(0);
				energy.delete();
				damage--;
			}
			return 0;
		} else {
			if (getShieldStrength() > 0) {
				Log.log(getName() + " reduce the damage by " + getShieldStrength() + ".");
				damage = damage - getShieldStrength();
				clear();
			}
			return damage;
		}
	}
	public void loadWithEnergy(Nameable player, boolean isHeroic) {
		InternalAttachedThreatBase threat = getBlocker();
		if (threat != null) {
			threat.damage(player, this, isHeroic);
		} else {
			List<Energy> newEnergies = reactor.getEnergy(getMaximumEnergy() - energies.size());
			energies.addAll(newEnergies);
			if (isHeroic) {
				Energy energy = new Energy(map);
				energy.moveTo(location);
				energies.add(energy);
			}
			
			for (Energy energy: newEnergies) {
				energy.moveTo(location);
			}
			Log.log(player.getName() + " loads " + getName() + " with energy.");
		}
	}
	
	public void clear() {
		for (Energy energy: energies) {
			energy.delete();
		}
		energies.clear();
	}
	
	public String getName() {
		return name;
	}
	
	public ShieldBase(ReactorBase reactor, ThreatZones zone, boolean damaged) {
		super(reactor);
		
		String location;
		switch(zone) {
		case BLUE:
			location = "Blue: Shields";
			name = "blue shield";
			break;
		case RED:
			location = "Red: Shields";
			name = "red shield";
			break;
		case INTERNAL:
		case WHITE:
		default:
			location = "White: Shields (1)";
			name = "white shield";
			break;
		
		}
		this.location = map.findRegion(location);
		this.reactor = reactor;
		this.damaged = damaged;
		
		if (!damaged) {
			Energy energy = new Energy(map);
			energy.moveTo(this.location);
			energies.add(energy);
		};
	}

}
