package ship;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.swing.KeyStroke;

import spaceAlert.CardAction;
import spaceAlert.Log;
import spaceAlert.PlayerCardsOrdered;
import spaceAlert.PlayersCards;
import spaceAlert.ResetableBase;
import spaceAlert.ResolutionThread;
import spaceAlert.Space;
import spaceAlert.VassalHelper;
import threats.InternalMovingThreatBase;
import threats.InternalAttachedThreatBase;
import threats.ThreatBase;
import threats.ThreatCompareETA;
import threats.ThreatDamageSpecial;
import threats.ThreatZones;
import yellowThreats.E2_02;

import VASSAL.command.Command;
import VASSAL.command.RemovePiece;
import VASSAL.counters.GamePiece;
import VASSAL.counters.Stack;

public class Ship extends ResetableBase {
	public List<ThreatZones> getDoubleDamageZones() {
		return doubleDamageZones;
	}

	private DamageGenerator damageGenerator;
	public DamageGenerator getDamageGenerator() {
		return damageGenerator;
	}
	
	private List<Player> players = new ArrayList<Player>();
	public List<Player> getPlayers() {
		return players;
	}
	
	private Optics optics;
	public Optics getOptics() {
		return optics;
	}
	
	private Computer computer;
	public Computer getComputer() {
		return computer;
	}
	private Window window;
	public Window getWindow() {
		return window;
	}
	
	private Battlebots redBattleBots;
	public Battlebots getRedBattleBots() {
		return redBattleBots;
	}
	private Battlebots blueBattleBots;
	public Battlebots getBlueBattleBots() {
		return blueBattleBots;
	}
	
	private PrimaryReactor whiteReactor;
	public PrimaryReactor getWhiteReactor() {
		return whiteReactor;
	}
	private SecondaryReactor redReactor;
	public SecondaryReactor getRedReactor() {
		return redReactor;
	}
	private SecondaryReactor blueReactor;
	public SecondaryReactor getBlueReactor() {
		return blueReactor;
	}
	
	private PrimaryShield whiteShield;
	public PrimaryShield getWhiteShield() {
		return whiteShield;
	}
	private SecondaryShield redShield;
	public SecondaryShield getRedShield() {
		return redShield;
	}
	private SecondaryShield blueShield;
	public SecondaryShield getBlueShield() {
		return blueShield;
	}
	
	private PrimaryHeavyLaserCannon whiteHeavyLaser;
	public PrimaryHeavyLaserCannon getWhiteHeavyLaser() {
		return whiteHeavyLaser;
	}
	private SecondaryHeavyLaserCannon redHeavyLaser;
	public SecondaryHeavyLaserCannon getRedHeavyLaser() {
		return redHeavyLaser;
	}
	private SecondaryHeavyLaserCannon blueHeavyLaser;
	public SecondaryHeavyLaserCannon getBlueHeavyLaser() {
		return blueHeavyLaser;
	}
	private PulseCannon pulseCannon;
	public PulseCannon getPulseCannon() {
		return pulseCannon;
	}
	private LightLaserCannon redLightLaser;
	public LightLaserCannon getRedLightLaser() {
		return redLightLaser;
	}
	private LightLaserCannon blueLightLaser;
	public LightLaserCannon getBlueLightLaser() {
		return blueLightLaser;
	}
	
	private RocketThrower rocketThrower;
	public RocketThrower getRocketThrower() {
		return rocketThrower;
	}
	private Fighters fighters;
	public Fighters getFighters() {
		return fighters;
	}
	
	private boolean redElevatorAvailable = true;
	public boolean isRedElevatorAvailable() {
		return redElevatorAvailable;
	}
	public void setRedElevatorAvailable(boolean available) {
		redElevatorAvailable = available;
	}
	private boolean redElevatorDamaged = false;
	
	private boolean whiteElevatorAvailable = true;
	public boolean isWhiteElevatorAvailable() {
		return whiteElevatorAvailable;
	}
	public void setWhiteElevatorAvailable(boolean available) {
		whiteElevatorAvailable = available;
	}
	private boolean whiteElevatorDamaged = false;
	
	private boolean blueElevatorAvailable = true;
	public boolean isBlueElevatorAvailable() {
		return blueElevatorAvailable;
	}
	public void setBlueElevatorAvailable(boolean available) {
		blueElevatorAvailable = available;
	}
	private boolean blueElevatorDamaged = false;
	
	private String destroyer;
	public String getDestroyer() {
		return destroyer;
	}
	private boolean destroyed = false;
	public void destroy(String destroyer) {
		Log.log("Ship has been destroyed by " + destroyer + ".");
		this.destroyer = destroyer;
		destroyed = true;
	}
	public boolean isDestroyed() {
		return destroyed;
	}
	
	private List<ThreatZones> doubleDamageZones = new LinkedList<ThreatZones>();
	public void addDoubleDamageZones(List<ThreatZones> newZones) {
		doubleDamageZones.addAll(newZones);
	}
	public void removeDoubleDamageZones(List<ThreatZones> repairedZones) {
		doubleDamageZones.removeAll(repairedZones);
	}
	private boolean targetedByScout = false;
	private String scoutName;
	public void targetByScout(E2_02 scout) {
		if ((scout != null)
		&& scout.isAppeared()
		&& !scout.isDestroyed()) {
			targetedByScout = true;
			scoutName = scout.getName();
		}
	}
	public void untargetByScout() {
		targetedByScout = false;
	}
			
	private InitialDamagedSystems cleanUp(boolean isCampaign) {
		if (!isCampaign) {
			//repair all damage
			GamePiece allPieces[] = map.getAllPieces();
			for (GamePiece piece : allPieces) {
				piece.keyEvent(KeyStroke.getKeyStroke("control R"));
			}
		}
		List<ShipDamage> damagedSystems = new LinkedList<ShipDamage>();
		
		damageGenerator = null;
		players.clear();
		optics = null;
		window = null;
		computer = null;
		redBattleBots = null;
		blueBattleBots = null;
		whiteReactor = null;
		redReactor = null;
		blueReactor = null;
		whiteShield = null;
		redShield = null;
		blueShield = null;
		whiteHeavyLaser = null;
		redHeavyLaser = null;
		blueHeavyLaser = null;
		pulseCannon = null;
		redLightLaser = null;
		blueLightLaser = null;
		rocketThrower = null;
		fighters = null;
		
		int numberOfBattlebotSquads = 0;
				
		GamePiece[] allPieces = map.getAllPieces();
		for (GamePiece piece : allPieces) {
			String location = map.locationName(piece.getPosition());
			if (location.startsWith("Red:")
			|| location.startsWith("White:")
			|| location.startsWith("Blue:")) {
				if (piece instanceof Stack) {
					Stack stack = (Stack)piece;
					for (Iterator<GamePiece> iter = stack.getPiecesIterator(); iter.hasNext();) {
						numberOfBattlebotSquads = checkAndDeleteGamePiece(damagedSystems, numberOfBattlebotSquads, iter.next());
					}
				} else {
					numberOfBattlebotSquads = checkAndDeleteGamePiece(damagedSystems, numberOfBattlebotSquads, piece);
				}
			}
		}
		if (!isCampaign) {
			numberOfBattlebotSquads = 2;
		}
		return new InitialDamagedSystems(damagedSystems, numberOfBattlebotSquads);
	}
	
	private int checkAndDeleteGamePiece(List<ShipDamage> damagedSystems,
			int numberOfBattlebotSquads, GamePiece piece) {
		String prop = VassalHelper.getPropertyFromPieceInStack(piece, "class");
		
		if (!"Damage".equals(prop)) {
			
			if ("Battlebots".equals(prop)) {
				int damaged = Integer.valueOf((String) piece.getProperty("damaged"));
				if (damaged == 0) {
					numberOfBattlebotSquads++;
				}
			} else if ("Player".equals(prop)) {
				int followedByBattlebots = Integer.valueOf((String) piece.getProperty("battlebots"));
				if (followedByBattlebots == 1) {
					numberOfBattlebotSquads++;
				}
			}
			
			Command c = new RemovePiece(piece);
			c.execute();
		} else {
			damagedSystems.add(ShipDamage.getDamage(piece.getName()));
		}
		return numberOfBattlebotSquads;
	}
	
	public boolean initialize(PlayerCardsOrdered playerOrdered, boolean isCampaign) {
		InitialDamagedSystems damagedSystems = cleanUp(isCampaign);
		if (damagedSystems.isCanceled()) {
			return false;
		}
		
		damageGenerator = new DamageGenerator(damagedSystems.getDamagedSystems());
		
		for (PlayersCards playerCards: playerOrdered) {
			players.add(new Player(playerCards, map));
		}
				
		optics = new Optics(this);
		
		whiteReactor = new PrimaryReactor(this, damagedSystems.isWhiteReactorDamaged());
		redReactor = new SecondaryReactor(whiteReactor, ThreatZones.RED, damagedSystems.isRedReactorDamaged());
		blueReactor = new SecondaryReactor(whiteReactor, ThreatZones.BLUE, damagedSystems.isBlueReactorDamaged());

		whiteShield = new PrimaryShield(whiteReactor, damagedSystems.isWhiteShieldDamaged());
		redShield = new SecondaryShield(redReactor, ThreatZones.RED, damagedSystems.isRedShieldDamaged());
		blueShield = new SecondaryShield(blueReactor, ThreatZones.BLUE, damagedSystems.isBlueShieldDamaged());
		
		whiteHeavyLaser = new PrimaryHeavyLaserCannon(whiteReactor, damagedSystems.isWhiteHeavyLaserCannonDamaged());
		redHeavyLaser = new SecondaryHeavyLaserCannon(redReactor, "Red: Heavy Laser Cannon", damagedSystems.isRedHeavyLaserCannonDamaged());
		blueHeavyLaser = new SecondaryHeavyLaserCannon(blueReactor, "Blue: Heavy Laser Cannon", damagedSystems.isBlueHeavyLaserCannonDamaged());
		pulseCannon = new PulseCannon(whiteReactor, damagedSystems.isPulseCannonDamaged());
		redLightLaser = new LightLaserCannon(this, "Red: Light Laser Cannon", damagedSystems.isRedLightLaserCannonDamaged());
		blueLightLaser = new LightLaserCannon(this, "Blue: Light Laser Cannon", damagedSystems.isBlueLightLaserCannonDamaged());
	
		rocketThrower = new RocketThrower(this);
		fighters = new Fighters(optics);

		redBattleBots = new Battlebots(map, true, damagedSystems.isRedBattlebotSquadDestroyed());
		blueBattleBots = new Battlebots(map, false, damagedSystems.isBlueBattlebotSquadDestroyed());
		
		computer = new Computer(this);
		window = new Window(optics);
		
		if (damagedSystems.isRedStructureDamaged()) {
			doubleDamageZones.add(ThreatZones.RED);
		}
		if (damagedSystems.isWhiteStructureDamaged()) {
			doubleDamageZones.add(ThreatZones.WHITE);
		}
		if (damagedSystems.isBlueStructureDamaged()) {
			doubleDamageZones.add(ThreatZones.BLUE);
		}
		redElevatorDamaged = damagedSystems.isRedElevatorDamaged();
		whiteElevatorDamaged = damagedSystems.isWhiteElevatorDamaged();
		blueElevatorDamaged = damagedSystems.isBlueElevatorDamaged();
		
		return true;
	}
	
	public void playerAction(int currentTime) {
		for (Player player : players) {
			if (!player.isUnconscious()) {
				CardAction action = player.getActionAt(currentTime);
				
				if (player.getLocation() == ShipZones.FIGHTERS) {
					fighters.doPlayerAction(player, action);
				} else {
					switch (action) {
					case MOVE_TO_BLUE:
					case MOVE_TO_RED:
					case SWITCH_DECK:
						doPlayerMoveAction(player, action);
						ResolutionThread.getInstance().doSleep();
						break;
					case PRESS_A:
						doPlayerAPressedAction(player, false);
						ResolutionThread.getInstance().doSleep();
						break;
					case PRESS_B:
						doPlayerBPressedAction(player, false);
						ResolutionThread.getInstance().doSleep();
						break;
					case PRESS_C:
						doPlayerCPressedAction(player);
						ResolutionThread.getInstance().doSleep();
						break;
					case ACTIVATE_BOTS:
						doPlayerActivatedBotsAction(player, false);
						ResolutionThread.getInstance().doSleep();
						break;
					case HEROIC_A:
						doPlayerAPressedAction(player, true);
						ResolutionThread.getInstance().doSleep();
						break;
					case HEROIC_B:
						doPlayerBPressedAction(player, true);
						ResolutionThread.getInstance().doSleep();
						break;
					case HEROIC_BOT:
						doPlayerActivatedBotsAction(player, true);
						ResolutionThread.getInstance().doSleep();
						break;
					case HEROIC_MOVE_TO_BLUE_LOWER:
						Movement.heroicMoveTo(player, ShipZones.BLUE_LOWER);
						ResolutionThread.getInstance().doSleep();
						break;
					case HEROIC_MOVE_TO_BLUE_UPPER:
						Movement.heroicMoveTo(player, ShipZones.BLUE_UPPER);
						ResolutionThread.getInstance().doSleep();
						break;
					case HEROIC_MOVE_TO_RED_LOWER:
						Movement.heroicMoveTo(player, ShipZones.RED_LOWER);
						ResolutionThread.getInstance().doSleep();
						break;
					case HEROIC_MOVE_TO_RED_UPPER:
						Movement.heroicMoveTo(player, ShipZones.RED_UPPER);
						ResolutionThread.getInstance().doSleep();
						break;
					case HEROIC_MOVE_TO_WHITE_LOWER:
						Movement.heroicMoveTo(player, ShipZones.WHITE_LOWER);
						ResolutionThread.getInstance().doSleep();
						break;
					case HEROIC_MOVE_TO_WHITE_UPPER:
						Movement.heroicMoveTo(player, ShipZones.WHITE_UPPER);
						ResolutionThread.getInstance().doSleep();
						break;
					case NOTHING:
					default:
						Log.log(player.getName() + " does nothing.");
						ResolutionThread.getInstance().doSleepBetweenPhases();
					}
				}
			}
		}
	}
	
	private void doPlayerActivatedBotsAction(Player player, boolean isHeroic) {
		if (player.hasBattlebots()
		&& !player.getBattlebots().isDamaged()) {
			List<InternalMovingThreatBase> internalThreats = new LinkedList<InternalMovingThreatBase>();
			for (ThreatBase threat : Space.getInstance().getThreats()) {
				if (threat instanceof InternalMovingThreatBase) {
					InternalMovingThreatBase internalThreat = (InternalMovingThreatBase) threat;
					if (internalThreat.getLocation() == player.getLocation()) {
						internalThreats.add((InternalMovingThreatBase)threat);
					};
				}
			}
			if (internalThreats.size() > 0) {
				Collections.sort(internalThreats, new ThreatCompareETA());
				internalThreats.get(0).damageBy(player, isHeroic);
			}			
		}
	}
	
	private void doPlayerCPressedAction(Player player) {
		InternalAttachedThreatBase threat;
		switch (player.getLocation()) {
		case BLUE_LOWER:
			rocketThrower.fire(player, false);
			break;
		case BLUE_UPPER:
			threat = blueBattleBots.getBlocker();
			if (threat != null) {
				threat.damage(player, blueBattleBots, false);
			} else {
				if (player.hasBattlebots()) {
					player.getBattlebots().repair();
					Log.log(player.getName() + " re-activates its battlebot squad.");
				} else {
					player.addBattlebots(blueBattleBots);
				}
			}
			break;
		case RED_LOWER:
			threat = redBattleBots.getBlocker();
			if (threat != null) {
				threat.damage(player, redBattleBots, false);
			} else {
				if (player.hasBattlebots()) {
					player.getBattlebots().repair();
					Log.log(player.getName() + " re-activates its battlebot squad.");
				} else {
					player.addBattlebots(redBattleBots);
				}
			}
			break;
		case RED_UPPER:
			fighters.man(player);
			break;
		case WHITE_LOWER:
			window.lookThrough(player);
			break;
		case WHITE_UPPER:
			computer.maintain(player);
			break;
		default:
			break;
		
		}
	}
	
	private void doPlayerBPressedAction(Player player, boolean isHeroic) {
		switch (player.getLocation()) {
		case BLUE_LOWER:
			blueReactor.getEnergyFromPrimaryReactor(player, isHeroic);
			break;
		case BLUE_UPPER:
			blueShield.loadWithEnergy(player, isHeroic);
			break;
		case RED_LOWER:
			redReactor.getEnergyFromPrimaryReactor(player, isHeroic);
			break;
		case RED_UPPER:
			redShield.loadWithEnergy(player, isHeroic);
			break;
		case WHITE_LOWER:
			whiteReactor.createEnergy(player, isHeroic);
			break;
		case WHITE_UPPER:
			whiteShield.loadWithEnergy(player, isHeroic);
			break;
		default:
			break;
		}
	}
	
	private void doPlayerAPressedAction(Player player, boolean isHeroic) {
		switch (player.getLocation()) {
		case BLUE_LOWER:
			blueLightLaser.fire(player, isHeroic);
			break;
		case BLUE_UPPER:
			blueHeavyLaser.fire(player, isHeroic);
			break;
		case RED_LOWER:
			redLightLaser.fire(player, isHeroic);
			break;
		case RED_UPPER:
			redHeavyLaser.fire(player, isHeroic);
			break;
		case WHITE_LOWER:
			pulseCannon.fire(player, isHeroic);
			break;
		case WHITE_UPPER:
			whiteHeavyLaser.fire(player, isHeroic);
			break;
		default:
			break;
		}
	}
	
	private void doPlayerMoveAction(Player player, CardAction action) {
		Movement.moveTo(player, action);
	}
	
	public boolean isMovementAllowed(ShipZones currentLocation, ShipZones targetLocation) {
		return isMovementAllowed(currentLocation.getThreatZone(), targetLocation.getThreatZone());
	}
	
	public boolean isMovementAllowed(ThreatZones currentLocation, ThreatZones targetLocation) {
		return true;
	}
	
	public void shieldedDamage(ThreatZones zone, int damage) {
		shieldedDamage(zone, damage, ThreatDamageSpecial.NOTHING);
	}
	
	public void shieldedDamage(ThreatZones zone, int damage, ThreatDamageSpecial special) {
		if (targetedByScout
		&& (special != ThreatDamageSpecial.PHASING)) { //increase damage by threats when targeted by a scout
			Log.log(scoutName + " increases the attack by 1");
			damage++;
		}
		
		int originalDamage = damage;
		switch (zone) {
		case BLUE:
			damage = blueShield.recieveDamage(damage);
			break;
		case RED:
			damage = redShield.recieveDamage(damage);
			break;
		case WHITE:
			damage = whiteShield.recieveDamage(damage);
			break;
			
		default:
			break;
		}
		switch (special) {
		case DOUBLED_DAMAGE:
			damage *= 2;
			break;
		case PLASMA:
			if (originalDamage == damage) {
				knockOutPlayersIn(zone);
			}
			break;
		default:
			break;		
		}
		damage(zone, damage);
	}
	
	public void damage(ThreatZones zone, int damage) {
		if (doubleDamageZones.contains(zone)) {
			damage *= 2;
		}
		while (damage > 0) {
			if (!damage(zone)) {
				//ship has been destroyed
				break;
			}
			damage--;
		}
	}
	
	  /**
	   * Adds one damage to the ship
	   * @return ship has survived?
	   */
	private boolean damage(ThreatZones zone) {
		ShipDamage damage = damageGenerator.getDamage(zone);
		damageSystems(damage);
		
		GamePiece damagePiece = damageGenerator.getDamagePiece(damage);
		if (damagePiece == null) {
			return false;
		} else {
			map.addPiece(damagePiece);
			damagePiece.keyEvent(KeyStroke.getKeyStroke("alt control shift P"));
			return true;
		}
	}
	private void damageSystems(ShipDamage damage) {
		switch (damage) {
		case BLUE_ELEVATOR:
			blueElevatorDamaged = true;
			break;
		case RED_ELEVATOR:
			redElevatorDamaged = true;
			break;
		case WHITE_ELEVATOR:
			whiteElevatorDamaged = true;
			break;
			
		case BLUE_HEAVY_LASER_CANNON:
			blueHeavyLaser.damage();
			break;
		case BLUE_LIGHT_LASER_CANNON:
			blueLightLaser.damage();
			break;
		case BLUE_REACTOR:
			blueReactor.damage();
			break;
		case BLUE_SHIELD:
			blueShield.damage();
			break;
		case RED_HEAVY_LASER_CANNON:
			redHeavyLaser.damage();
			break;
		case RED_LIGHT_LASER_CANNON:
			redLightLaser.damage();
			break;
		case RED_REACTOR:
			redReactor.damage();
			break;
		case RED_SHIELD:
			redShield.damage();
			break;
		case WHITE_HEAVY_LASER_CANNON:
			whiteHeavyLaser.damage();
			break;
		case WHITE_PULSE_CANNON:
			pulseCannon.damage();
			break;
		case WHITE_REACTOR:
			whiteReactor.damage();
			break;
		case WHITE_SHIELD:
			whiteShield.damage();
			break;
		
		case BLUE_STRUCTURE:
		case WHITE_STRUCTURE:
		case RED_STRUCTURE:
			break;
		
		case DESTROYED:
		default:
			destroyed = true;
			break;
		}
	}
	
	private void knockOutPlayersIn(ThreatZones zone) {
		switch (zone) {
		case BLUE:
			knockOutPlayersIn(ShipZones.BLUE_LOWER);
			knockOutPlayersIn(ShipZones.BLUE_UPPER);
			break;
		case RED:
			knockOutPlayersIn(ShipZones.RED_LOWER);
			knockOutPlayersIn(ShipZones.RED_UPPER);
			break;
		case WHITE:
			knockOutPlayersIn(ShipZones.WHITE_LOWER);
			knockOutPlayersIn(ShipZones.WHITE_UPPER);
			break;
		default:
			break;
		}
	}
	
	public void knockOutPlayersIn(ShipZones zone) {
		for (Player player: players) {
			if (player.getLocation() == zone) {
				player.knockOut();
			}
		}
	}
	
	public void delayPlayersIn(ShipZones zone) {
		for (Player player: players) {
			if (player.getLocation() == zone) {
				player.delay();
			}
		}
	}
	
	@Override
	public void reset() {
		super.reset();
		
		redElevatorAvailable = !redElevatorDamaged;
		whiteElevatorAvailable = !whiteElevatorDamaged;
		blueElevatorAvailable = !blueElevatorDamaged;
	}
	
	public Ship(Space space) {
		super(space);
		
		Movement.initialize(space, this);
	}
}
