package spaceAlert;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;

import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;

import VASSAL.build.AbstractConfigurable;
import VASSAL.build.Buildable;
import VASSAL.build.GameModule;
import VASSAL.build.module.GameComponent;
import VASSAL.build.module.Map;
import VASSAL.build.module.documentation.HelpFile;
import VASSAL.build.module.map.ImageSaver;
import VASSAL.command.Command;
import VASSAL.counters.GamePiece;
import VASSAL.preferences.Prefs;
import VASSAL.tools.ToolBarComponent;

public class Main extends AbstractConfigurable implements
		GameComponent {
	private JMenuBar menu;
	
	private PlayersCards players[] = new PlayersCards[5];
	private Space space;
	private Map missionControl;
	
	private void reset(boolean isCampaign) {
		if (ResolutionThread.getInstance().isAlive()) {
			Log.error("Can't reset while resolution in progress");
		} else {
			GamePiece[] pieces = missionControl.getAllPieces();
			PlayerCardsOrdered playerOrdered = new PlayerCardsOrdered(pieces[0], players);
			space.initialize(playerOrdered, isCampaign);
		}
	}
	
	private void initialize(boolean isCampaign) {
		if (ResolutionThread.getInstance().isAlive()) {
			Log.error("Can't start a new resolution while resolution in progress");
		} else {
			for (PlayersCards player: players) {
				player.lockCards();
			}
			GamePiece[] pieces = missionControl.getAllPieces();
			
			ImageSaver imageSaver = null;
	//		List<Map> maps = GameModule.getGameModule().getAllDescendantComponentsOf(Map.class);
	//		for (Map map: maps) {
	//			if (map.getMapName().equals("Main Map")) {
	//				List<ImageSaver> imageSavers = map.getAllDescendantComponentsOf(ImageSaver.class);
	//				if (imageSavers.size() > 0) {
	//					imageSaver = imageSavers.get(0);
	//				}
	//			}
	//		}
	//		
			PlayerCardsOrdered playerOrdered = new PlayerCardsOrdered(pieces[0], players);
			if (!space.initialize(playerOrdered, isCampaign)) {
				return;
			}
			space.reset();
	
			ResolutionThread missionControlThread = ResolutionThread.getInstance();
			missionControlThread.initialize(pieces[0], space, imageSaver);
			Prefs prefs = GameModule.getGameModule().getPrefs();
			missionControlThread.setDefaultSleep((Integer) prefs.getValue("defaultSleep"));
			missionControlThread.setTinySleep((Integer) prefs.getValue("tinySleep"));
			missionControlThread.start();
		}
	}
	
	private void pauseThread() {
		ResolutionThread resolutionThreat = ResolutionThread.getInstance();
		if (resolutionThreat.isAlive()) {
			if (!resolutionThreat.isPaused()) {
				resolutionThreat.pauseThread();
			} else {
				Log.log("---Resolution is continued---");
			      synchronized(resolutionThreat) {
			    	  resolutionThreat.notify();
			      }
			}
		}
	}
	
	private void stopThread() {
		ResolutionThread resolutionThread = ResolutionThread.getInstance();
		if (resolutionThread.isAlive()) {
			resolutionThread.stopThread();
		}
	}
	
	public void addTo(Buildable parent) {
		ToolBarComponent mod = (ToolBarComponent)parent;
		
		menu = new JMenuBar();
		mod.getToolBar().addSeparator();
		mod.getToolBar().add(menu);
		mod.getToolBar().addSeparator();
		
        JMenu menuInitialize = new JMenu("Launch");
        JMenuItem item1 = new JMenuItem("normal");
        item1.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				initialize(false);
			}});
        menuInitialize.add(item1);
        JMenuItem item2 = new JMenuItem("as campaign");        
        item2.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				initialize(true);
			}});
        menuInitialize.add(item2);

        JMenu menuReset = new JMenu("Reset");
        item1 = new JMenuItem("everything");
        item1.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				reset(false);
			}});
        menuReset.add(item1);
        item2 = new JMenuItem("without repairs");        
        item2.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				reset(true);
			}});
        menuReset.add(item2);
        
        JMenu menuControl = new JMenu("Control");
        item1 = new JMenuItem("Pause/Continue");
        item1.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				pauseThread();
			}});
        menuControl.add(item1);
        item2 = new JMenuItem("Stop");        
        item2.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				stopThread();
			}});
        menuControl.add(item2);
        
        menu.add(menuInitialize);
        menu.add(menuReset);
        menu.add(menuControl);

		GameModule.getGameModule().getGameState().addGameComponent(this);
	}

	public void removeFrom(Buildable parent) {
		ToolBarComponent mod = (ToolBarComponent)parent;
		
		mod.getToolBar().remove(menu);
		
		GameModule.getGameModule().getGameState().removeGameComponent(this);
	}
	
	public void setup(boolean gameStarting) {
		if (gameStarting) {
	    	List<PlayersCards> playersCards = GameModule.getGameModule().getAllDescendantComponentsOf(PlayersCards.class);
	    	if (playersCards.size() != 5) {
	    		throw new RuntimeException("Didn't found all five player cards");
	    	} else {
	    		for (int i = 0; i < 5; i++) {
	    			players[i] = playersCards.get(i);
	    		}
	    	}
			Map mainMap = null;
			List<Map> maps = Map.getMapList();
	    	for (Map map : maps) {
	    		if (map.getMapName().equals("Main Map")) {
	    			mainMap = map;
	    		} else if (map.getMapName().equals("Mission Control Map")) {
	    			missionControl = map;
	    		}
	    	}
	    	if ((mainMap == null)
	    	|| (missionControl == null)) {
	    		throw new RuntimeException("Didn't found required maps");	    		
	    	} else {
		    	space = new Space(mainMap);
	    	}
		} else {
			ResolutionThread resolutionThread = ResolutionThread.getInstance();
			if (resolutionThread.isAlive()) {
				resolutionThread.stopThread();
				try {
					resolutionThread.join();
				} catch (InterruptedException e) {}
			}
		}
	}

	public HelpFile getHelpFile() {
		return null;
	}

	public Class[] getAllowableConfigureComponents() {
		return new Class[0];
	}

	public Command getRestoreCommand() {
		return null;
	}

	@Override
	public String[] getAttributeDescriptions() {
		// TODO Auto-generated method stub
		return new String[]{};
	}

	@Override
	public Class<?>[] getAttributeTypes() {
		// TODO Auto-generated method stub
		return new Class[]{};
	}

	@Override
	public String[] getAttributeNames() {
		// TODO Auto-generated method stub
		 return new String[]{};
	}

	@Override
	public void setAttribute(String key, Object value) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public String getAttributeValueString(String key) {
		// TODO Auto-generated method stub
		return null;
	}

}
