package spaceAlert;

import javax.swing.KeyStroke;

import VASSAL.build.AbstractConfigurable;
import VASSAL.build.Buildable;
import VASSAL.build.GameModule;
import VASSAL.build.module.GameComponent;
import VASSAL.build.module.Map;
import VASSAL.build.module.map.boardPicker.board.Region;
import VASSAL.command.Command;
import VASSAL.counters.GamePiece;
import VASSAL.counters.Stack;

public class PlayersCards extends AbstractConfigurable implements GameComponent {
	private Map ownerMap;
	private PlayerColor color = PlayerColor.YELLOW;
	public PlayerColor getColor() {
		return color;
	}
	private GamePiece actionCards[] = new GamePiece[13];
	
	private void clearCards() {
		for (int i = 0; i < 13; i++) {
			actionCards[i] = null;
		}
	}
	
	public void lockCards() {
		clearCards();
		
		GamePiece[] stacks = ownerMap.getAllPieces();
		for (GamePiece stack : stacks) {
			GamePiece card = null;
			int i = Integer.parseInt(ownerMap.locationName(stack.getPosition())); //currents card location
			if ((i > 0) 
			&& (i < 13)) {
				if (stack instanceof Stack) {
					if (((Stack) stack).getPieceCount() > 1) {
						throw new RuntimeException("Multiple cards at time " + i + " | " + color);
					} else {
						if (((Stack) stack).getPieceCount() > 0) {
							card = ((Stack) stack).getPieceAt(0);
						}
					}
				} else {
					card = stack;
				}
				if (card != null) {
					actionCards[i - 1] = card;
				}
			}
		}
//		Log.debug(color + " cards:");
//		for (int i = 1; i < 13; i++) {
//			if (actionCards[i - 1] != null) {
//				Log.debug(i + ": " + getActionAt(i) + " | " + actionCards[i - 1].getName());
//			} else {
//				Log.debug(i + ": " + getActionAt(i) + " | NULL");
//			}
//		}
//		Log.debug("----");
	}
	
	public void unmaskCardAt(int time) {
		GamePiece card = actionCards[time -1];
		if (card != null) {
			card.keyEvent(KeyStroke.getKeyStroke("control M"));
			ownerMap.repaint();
		}
	}
	
	public CardAction getActionAt(int time) {
		GamePiece card = actionCards[time -1];
		if (card == null) {
			return CardAction.NOTHING;
		} else {
			String name = card.getName();
			if (name.startsWith("A -")) {
				return CardAction.PRESS_A;
			} else if (name.startsWith("B -")) {
				return CardAction.PRESS_B;
			} else if (name.startsWith("C -")) {
				return CardAction.PRESS_C;
			} else if (name.startsWith("Bot -")) {
				return CardAction.ACTIVATE_BOTS;
			} else if (name.startsWith("Blue -")) {
				return CardAction.MOVE_TO_BLUE;
			} else if (name.startsWith("Red -")) {
				return CardAction.MOVE_TO_RED;
			} else if (name.startsWith("Yellow -")) {
				return CardAction.SWITCH_DECK;
			} else if (name.startsWith("Heroic -")) {
				return getHeroicActionBy(name);
			} else {
				return CardAction.NOTHING;
			}
		}
	}
	
	private CardAction getHeroicActionBy(String name) {
		name = name.substring(9);
		if (name.startsWith("A -")) {
			return CardAction.HEROIC_A;
		} else if (name.startsWith("B -")) {
			return CardAction.HEROIC_B;
		} else if (name.startsWith("Bot -")) {
			return CardAction.HEROIC_BOT;
		} else if (name.startsWith("Blue Lower -")) {
			return CardAction.HEROIC_MOVE_TO_BLUE_LOWER;
		} else if (name.startsWith("Blue Upper -")) {
			return CardAction.HEROIC_MOVE_TO_BLUE_UPPER;
		} else if (name.startsWith("White Lower -")) {
			return CardAction.HEROIC_MOVE_TO_WHITE_LOWER;
		} else if (name.startsWith("White Upper -")) {
			return CardAction.HEROIC_MOVE_TO_WHITE_UPPER;
		} else if (name.startsWith("Red Lower -")) {
			return CardAction.HEROIC_MOVE_TO_RED_LOWER;
		} else if (name.startsWith("Red Upper -")) {
			return CardAction.HEROIC_MOVE_TO_RED_UPPER;
		} else {
			return CardAction.NOTHING;
		}
	}
	
	public void delayAt(int time) {
		if ((0 < time)
		&& (time < 13)) {
			Log.log(color.toString() + " gets delayed at " + time);
			delayWithoutLog(time);
		}
	}
	
	protected void delayWithoutLog(int time) {
		if ((0 < time)
		&& (time < 13)) {
			if (actionCards[time - 1] != null) { //is there a card at given time?
				if (actionCards[time] != null) { //is there a following card?
					delayWithoutLog(time + 1);
				}
				Region targetRegion = ownerMap.findRegion(String.valueOf(time + 1));
				ownerMap.placeOrMerge(actionCards[time - 1], targetRegion.getOrigin());
				
				actionCards[time] = actionCards[time - 1];
				actionCards[time - 1] = null;
			}
		}
	}
		

	private static final String COLOR = "color";
	@Override
	public void setAttribute(String key, Object value) {
		if (value != null) {
			if (COLOR.equals(key)) {
				String color = String.valueOf(value);
				
				if (color.equals("blue")) {
					this.color = PlayerColor.BLUE;
				} else if (color.equals("green")) {
					this.color = PlayerColor.GREEN;
				} else if (color.equals("purple")) {
					this.color = PlayerColor.PURPLE;
				} else if (color.equals("red")) {
					this.color = PlayerColor.RED;
				} else { //be brutal and use yellow as default
					this.color = PlayerColor.YELLOW;
				}
			}
		}
	}
	@Override
	public String[] getAttributeNames() {
		return new String[]{COLOR};
	}
	@Override
	public String[] getAttributeDescriptions() {
		return new String[]{"Player's color: "};
	}
	@Override
	public Class[] getAttributeTypes() {
		return new Class[]{String.class};
	}
	@Override
	public String getAttributeValueString(String key) {
		if (COLOR.equals(key)) {
			switch (color) {
			case BLUE:
				return "blue";
			case GREEN:
				return "green";
			case PURPLE:
				return "purple";
			case RED:
				return "red";
			default:
				return "yellow";
			}
		} else {
			return null;
		}
	}
	
	public void addTo(Buildable parent) {
		ownerMap = (Map)parent;
		
		GameModule mod = GameModule.getGameModule();
		mod.getGameState().addGameComponent(this);
	}
	public void removeFrom(Buildable parent) {
		GameModule mod = GameModule.getGameModule();
		mod.getGameState().removeGameComponent(this);
	}
	
	public VASSAL.build.module.documentation.HelpFile getHelpFile() {
		return null;
	}
	
	public Class[] getAllowableConfigureComponents() {
		return new Class[0];
	}
	
	public void setup(boolean gameStarting) {
	
	}
	
	public Command getRestoreCommand() {
		return null;
	}
}
