package spaceAlert;

import javax.swing.JOptionPane;

import threats.ThreatBase;
import whiteThreats.SE1_102;
import VASSAL.build.module.map.ImageSaver;
import VASSAL.counters.GamePiece;

public class ResolutionThread extends Thread {
	private static ResolutionThread instance;
	public static ResolutionThread getInstance() {
		if (instance == null) {
			instance = new ResolutionThread();
			instance.setUncaughtExceptionHandler(new ResolutionThreadExceptionHandler());
		}
		return instance;
	}
	
	private boolean stopped = false;
	public void stopThread() {
		stopped = true;
	}
	private boolean paused = false;
	public boolean isPaused() {
		return paused;
	}
	public void pauseThread() {
		paused = true;
	}
	private int tinySleep = 1000;
	public int getTinySleep() {
		return tinySleep;
	}
	public void setTinySleep(int value) {
		tinySleep = value;
	}
	private int defaultSleep = 2000;
	public int getDefaultSleep() {
		return defaultSleep;
	}
	public void setDefaultSleep(int value) {
		defaultSleep = value;
	}

	private GamePiece display;
	private Space space;
	private ImageSaver imageSaver;
	
	public void initialize(GamePiece display, Space space, ImageSaver imageSaver) {
		this.display = display;
		this.space = space;
		this.imageSaver = imageSaver;
	}
		
	protected ResolutionThread() {}
	
	public void doSleep() {
		if (!stopped) {
			try {
			    Thread.sleep( getDefaultSleep() );
			    if (paused) {
			    	Log.log("---Resolution is paused.---");
			    	synchronized(this) {
			    		this.wait();
			    	}
			    }
			    paused = false;
			} catch ( InterruptedException e ) { }
		}
	}
	
	public void doSleepBetweenPhases() {
		if (!stopped) {
			try {
				if (imageSaver != null) {
					imageSaver.writeMapAsImage();
				}
			    Thread.sleep( getTinySleep() );
			    if (paused) {
			    	Log.log("---Resolution is paused.---");
			    	synchronized(this) {
			    		this.wait();
			    	}
			    }
			    paused = false;
			} catch ( InterruptedException e ) { }
		}
	}

	@Override
	public void run() {	
		for (int time = 1; time < 14; time++) {
			if (stopped) {
				resetDisplay();
				return;
			}
			setWindowPhase(time);
			setDisplay(time);
			
			space.setCurrentTime(time);
			resolveComputer(time);
			threatAppears(time);		
			playerAction(time);
			computeThreatDamage(time);
			if (threatAction(time)) {
				break;
			}
			specialSE1_101Action(time);
		}
		if (!stopped) {
			if (space.getShip().isDestroyed()) {
			    missionFailed();
			} else {
				missionSuccessful();
			}
		}
	}

	private void missionSuccessful() {
		ScoreCalculator score = new ScoreCalculator(space);
		JOptionPane.showMessageDialog( null, "You survived! Your score is " + score.getOverallScore());
	}

	private void missionFailed() {
		JOptionPane.showMessageDialog( null, "Ship has been destroyed by " + space.getShip().getDestroyer() + ". You're dead..." );
	}

	private void specialSE1_101Action(int time) {
		if (time == 13) {
			for (ThreatBase threat: space.getThreats()) {
				if (threat instanceof SE1_102) {
					if (!threat.isDestroyed()
					&& !threat.isPassedBy()) {
						SE1_102 dimensionSpider = (SE1_102)threat;
						dimensionSpider.doZAction();
					}
				}
			}
		}
	}

	private void setWindowPhase(int time) {
		if (time == 1) {
			space.getShip().getWindow().setCurrentPhase(1);
		}
		if (time == 4) {
			space.getShip().getWindow().setCurrentPhase(2);
		}
		if (time == 8) {
			space.getShip().getWindow().setCurrentPhase(3);
		}
	}

	/**
	 * 
	 * @return true if ship has been destroyed
	 */
	private boolean threatAction(int time) {
		display.setProperty("action", 5);
		if (space.threatAction()) {
			return true;
		}
		doSleepBetweenPhases();
		return false;
	}

	private void computeThreatDamage(int time) {
		display.setProperty("action", 4);
		space.computeThreatDamage();
		space.reset();
		doSleepBetweenPhases();
	}

	private void playerAction(int time) {
		if (time < 13) {
			display.setProperty("action", 3);
			space.playerAction();
			doSleepBetweenPhases();
		} else if (time == 13) {
			space.getShip().getFighters().unMan();
		}
	}

	private void threatAppears(int time) {
		if (time < 9) {
			display.setProperty("action", 2);
			space.threatAppears();
			doSleepBetweenPhases();
		}
	}
	
	private void resolveComputer(int time) {
		if ((time == 3)
		|| (time == 6)
		|| (time == 10)) {
			display.setProperty("action", 1);
			if (space.resolveComputer()) {
				display.setProperty("computer", 2);
				doSleepBetweenPhases();
			} else {
				doSleep();
			}
		}
	}

	private void setDisplay(int time) {
		Log.log("------Round--" + time + "------");
		display.setProperty("round", time);
		if (time < 9) {
			display.setProperty("threatAppear", time);
		} else {
			display.setProperty("threatAppear", 0);
		}
		if (time < 13) {
			display.setProperty("playersAction", time);
		} else {
			display.setProperty("playersAction", 0);
		}
		if ((time == 3)
		|| (time == 6)
		|| (time == 10)) {
			display.setProperty("computer", 1);
		} else {
			display.setProperty("computer", 0);
		}
	}
	
	private void resetDisplay() {
		display.setProperty("action", 0);
	}

}
