package spaceAlert;

import ship.Player;
import ship.Ship;
import threats.ThreatBase;
import threats.ThreatZones;

public class ScoreCalculator {
	private int threatScore;
	private int visualConfirmationScore;
	public int getScore() {
		return threatScore + visualConfirmationScore;
	}
	private int malusByPlayer;
	private int malusByDamage;
	public int getMalus() {
		return malusByPlayer + malusByDamage;
	}
	public int getOverallScore() {
		return getScore() - getMalus();
	}
	public boolean isPerfectMission() {
		return getMalus() == 0;
	}
	
	public ScoreCalculator(Space space) {
		threatScore = 0;
		Ship ship = space.getShip();
		for (ThreatBase threat: space.getThreats()) {
			threatScore += threat.getScore();
		}
		visualConfirmationScore = ship.getWindow().getScore();
		
		malusByPlayer = 0;
		for (Player player: ship.getPlayers()) {
			if (player.isUnconscious()) {
				malusByPlayer += 2;
			}
		}
		if (ship.getBlueBattleBots().isDamaged()) {
			malusByPlayer += 1;
		}
		if (ship.getRedBattleBots().isDamaged()) {
			malusByPlayer += 1;
		}
		
		malusByDamage = 0;
		int damageInZoneRed = ship.getDamageGenerator().getDamageCountIn(ThreatZones.RED);
		int damageInZoneWhite = ship.getDamageGenerator().getDamageCountIn(ThreatZones.WHITE);
		int damageInZoneBlue = ship.getDamageGenerator().getDamageCountIn(ThreatZones.BLUE);
		int damageSum = damageInZoneRed + damageInZoneWhite + damageInZoneBlue;
		if (damageSum > 0) {
			malusByDamage += damageSum;
			malusByDamage += Math.max(damageInZoneBlue, Math.max(damageInZoneWhite, damageInZoneRed));
		}
				
		Log.log("------Mission Result------");
		Log.log("Score from threats: " + threatScore);
		Log.log("Score from visual confirmations: " + visualConfirmationScore);
		Log.log("Malus from players and battlebot squads: " + malusByPlayer);
		Log.log("Malus from ship damage: " + malusByDamage);
		Log.log("-----------------");
		Log.log("Overall: " + (getScore() - getMalus()));
	}

}
