package spaceAlert;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;

import javax.swing.KeyStroke;

import ship.Fighters;
import ship.Player;
import ship.PulseCannon;
import ship.RocketThrower;
import ship.Ship;
import ship.ShipZones;
import ship.WeaponBase;
import ship.WeaponRequiringReactorBase;
import threats.ExternalThreatBase;
import threats.ThreatBase;
import threats.ThreatCompareDistance;
import threats.ThreatCompareETA;
import threats.ThreatGenerator;
import threats.ThreatZones;
import trajectoryBoards.TrajectoryBoardBase;
import trajectoryBoards.TrajectoryBoardGenerator;
import yellowThreats.SE2_02;

import VASSAL.build.module.Map;
import VASSAL.command.Command;
import VASSAL.command.RemovePiece;
import VASSAL.counters.GamePiece;
import VASSAL.counters.Stack;

public class Space extends ResetableBase{
	private static Space instance;
	public static Space getInstance() {
		return instance;
	}
		
	private ThreatCompareETA threatCompareETA = new ThreatCompareETA();
	private ThreatCompareDistance threatCompareDistance = new ThreatCompareDistance();
	
	private TrajectoryBoardBase[] trajectoryBoards = new TrajectoryBoardBase[ThreatZones.values().length];
	private List<ThreatBase> threats = new ArrayList<ThreatBase>();
	public List<ThreatBase> getThreats() {
		return threats;
	}
	private List<ExternalThreatBase> threatsInZoneRed = new LinkedList<ExternalThreatBase>();
	private List<ExternalThreatBase> threatsInZoneWhite = new LinkedList<ExternalThreatBase>();
	private List<ExternalThreatBase> threatsInZoneBlue = new LinkedList<ExternalThreatBase>();
		
	private Ship ship;
	public Ship getShip() {
		return ship;
	}
	
	private static int currentTime;
	public static int getCurrentTime() {
		return currentTime;
	}
	public void setCurrentTime(int time) {
		currentTime = time;
	}
	
	public boolean initialize(PlayerCardsOrdered playerOrdered, boolean isCampaign) {
		boolean result = false;
		cleanUp();
		result = ship.initialize(playerOrdered, isCampaign);
		
		result &= getTrajectoryBoards();
		if (result) {
			createThreats();
		}
		return result;
	}

	private void createThreats() {
		threats.clear();
		threatsInZoneRed.clear();
		threatsInZoneWhite.clear();
		threatsInZoneBlue.clear();
		//find all threats
		GamePiece allPieces[] = map.getAllPieces();
		for (GamePiece piece : allPieces) {
			String prop = VassalHelper.getPropertyFromPieceInStack(piece, "class");
			if ("Threat".equals(prop)) {
				threats.add(ThreatGenerator.createThreat(ship, piece));
			}
		}
		for (ThreatBase threat: threats) {
			if (threat instanceof ExternalThreatBase) {
				switch (threat.getZone()) {
				case RED:
					threatsInZoneRed.add((ExternalThreatBase) threat);
					break;
				case WHITE:
					threatsInZoneWhite.add((ExternalThreatBase) threat);
					break;
				case BLUE:
					threatsInZoneBlue.add((ExternalThreatBase) threat);
					break;
				default:
					break;
				}				
			}
		}

		Collections.sort(threats, threatCompareETA);
		Collections.sort(threatsInZoneRed, threatCompareDistance);
		Collections.sort(threatsInZoneWhite, threatCompareDistance);
		Collections.sort(threatsInZoneBlue, threatCompareDistance);
	}

	private boolean getTrajectoryBoards() {
		trajectoryBoards[ThreatZones.RED.ordinal()] = null;
		trajectoryBoards[ThreatZones.WHITE.ordinal()] = null;
		trajectoryBoards[ThreatZones.BLUE.ordinal()] = null;
		trajectoryBoards[ThreatZones.INTERNAL.ordinal()] = null;
		
		GamePiece allPieces[] = map.getAllPieces();
		//find all trajectory boards
		for (GamePiece piece : allPieces) {
			String prop = VassalHelper.getPropertyFromPieceInStack(piece, "class");
			if ("Trajectory Board".equals(prop)) {
				String location = map.locationName(piece.getPosition());
				if ("Trajectory Board: Red".equals(location)) {
					if (trajectoryBoards[ThreatZones.RED.ordinal()] == null) {
						trajectoryBoards[ThreatZones.RED.ordinal()] = TrajectoryBoardGenerator.createTrajectoryBoard(piece, ThreatZones.RED);
					} else {
						Log.error("Multiple Trajectory Boards at the same location: " + location);
						return false;
					}
				} else if ("Trajectory Board: White".equals(location)) {
					if (trajectoryBoards[ThreatZones.WHITE.ordinal()] == null) {
						trajectoryBoards[ThreatZones.WHITE.ordinal()] = TrajectoryBoardGenerator.createTrajectoryBoard(piece, ThreatZones.WHITE);
					} else {
						Log.error("Multiple Trajectory Boards at the same location: " + location);
						return false;
					}
				} else if ("Trajectory Board: Blue".equals(location)) {
					if (trajectoryBoards[ThreatZones.BLUE.ordinal()] == null) {
						trajectoryBoards[ThreatZones.BLUE.ordinal()] = TrajectoryBoardGenerator.createTrajectoryBoard(piece, ThreatZones.BLUE);
					} else {
						Log.error("Multiple Trajectory Boards at the same location: " + location);
						return false;
					}
				} else if ("Trajectory Board: Internal".equals(location)) {
					if (trajectoryBoards[ThreatZones.INTERNAL.ordinal()] == null) {
						trajectoryBoards[ThreatZones.INTERNAL.ordinal()] = TrajectoryBoardGenerator.createTrajectoryBoard(piece, ThreatZones.INTERNAL);
					} else {
						Log.error("Multiple Trajectory Boards at the same location: " + location);
						return false;
					}
				} else {
					if (location.startsWith("Red:")
					|| location.startsWith("White:")
					|| location.startsWith("Blue:")
					|| location.startsWith("Space:")
					|| location.startsWith("Threat:")) {
						Log.error("Trajectory Board at wrong location: " + location);
						return false;
					}
				}
			}
		}
		if ((trajectoryBoards[ThreatZones.RED.ordinal()] == null)
		|| (trajectoryBoards[ThreatZones.WHITE.ordinal()] == null)
		|| (trajectoryBoards[ThreatZones.BLUE.ordinal()] == null)
		|| (trajectoryBoards[ThreatZones.INTERNAL.ordinal()] == null)) {
			Log.error("Not enough Trajectory Boards placed.");
			return false;
		}
		ThreatGenerator.trajectoryBoards = trajectoryBoards;
		return true;
	}

	private void cleanUp() {
		//place all non placed trajectory boards
		GamePiece allPieces[] = map.getAllPieces();
		for (GamePiece piece : allPieces) {
			String prop = VassalHelper.getPropertyFromPieceInStack(piece, "class");
			if ("Trajectory Board".equals(prop)) {
				piece.keyEvent(KeyStroke.getKeyStroke("control P"));
			}
		}
		map.repaint();
		
		clear(); // clears all childs of ResetableBase
		addChild(ship);
		
		//removes all wrong pieces on the space
		allPieces = map.getAllPieces();
		for (GamePiece piece : allPieces) {
			String location = map.locationName(piece.getPosition());
			if (location.startsWith("Trajectory Board:")
			|| location.startsWith("Threat:")
			|| location.startsWith("Space:")) {
				if (piece instanceof Stack) {
					Stack stack = (Stack)piece;
					for (Iterator<GamePiece> iter = stack.getPiecesIterator(); iter.hasNext();) {
						checkAndDeleteGamePiece(iter.next());
					}
				} else {
					checkAndDeleteGamePiece(piece);
				}
			}
		}
	}

	private void checkAndDeleteGamePiece(GamePiece piece) {
		String prop = VassalHelper.getPropertyFromPieceInStack(piece, "class");
		if (!"Trajectory Board".equals(prop)
		&& !"Threat".equals(prop)) {
			Command c = new RemovePiece(piece);
			c.execute();
		}
	}
	
	public void threatAppears() {
		ListIterator<ThreatBase> iterator = threats.listIterator();
		
		while (iterator.hasNext()) {
			ThreatBase threat = iterator.next();
			int eta = threat.getETA();
			if (eta == currentTime) {
				threat.appear();
				ResolutionThread.getInstance().doSleep();
			}
		}
	}
	
	public boolean resolveComputer() {
		if ((currentTime == 3)
		|| (currentTime == 6)
		|| (currentTime == 10)) {
			if (!ship.getComputer().isMaintained()) {
				Log.log("Computer was not maintained. Every player on the ship gets delayed.");
				//delay all player on the ship
				for (Player player: ship.getPlayers()) {
					if (player.getLocation() != ShipZones.FIGHTERS) {
						player.delayAt(currentTime);
					}
				}
			} else {
				return true;
			}
		}
		return false;
	}
	
	public void playerAction() {
		ship.playerAction(currentTime);
	}
	
	public void computeThreatDamage() {
		damageByPulseCannon();
		damageByLaser();
		damageByRocketThrower();
		damageByFighters();
		
		//remove destroyed threats
		for (Iterator<ExternalThreatBase> iter = threatsInZoneRed.iterator(); iter.hasNext(); ) {
			ExternalThreatBase threat = iter.next();
			if (threat.isDestroyed()) {
				iter.remove();
			}
		}
		for (Iterator<ExternalThreatBase> iter = threatsInZoneWhite.iterator(); iter.hasNext(); ) {
			ExternalThreatBase threat = iter.next();
			if (threat.isDestroyed()) {
				iter.remove();
			}
		}
		for (Iterator<ExternalThreatBase> iter = threatsInZoneBlue.iterator(); iter.hasNext(); ) {
			ExternalThreatBase threat = iter.next();
			if (threat.isDestroyed()) {
				iter.remove();
			}
		}		
	}

	private void damageByFighters() {
		Fighters fighters = ship.getFighters();
		if (fighters.hasFired()) {
			List<ExternalThreatBase> targets = new LinkedList<ExternalThreatBase>();
			targets.addAll(getTargetsInRange(fighters.getRange().getMinimumDistance(),
					fighters.getRange().getMaximumDistance(),
					threatsInZoneRed));
			targets.addAll(getTargetsInRange(fighters.getRange().getMinimumDistance(),
					fighters.getRange().getMaximumDistance(),
					threatsInZoneWhite));
			targets.addAll(getTargetsInRange(fighters.getRange().getMinimumDistance(),
					fighters.getRange().getMaximumDistance(),
					threatsInZoneBlue));
			
			if (targets.size() == 1) {
				fighters.setMultipleTargets(false);
				targets.get(0).damageBy(fighters);
			} else {
				fighters.setMultipleTargets(true);
				for (ExternalThreatBase target: targets) {
					target.damageBy(fighters);
				}
			}
			ResolutionThread.getInstance().doSleep();
		}
	}
	
	protected void damageByPulseCannon() {
		PulseCannon pulseCannon = ship.getPulseCannon();
		if (pulseCannon.hasFired()) {
			List<ExternalThreatBase> targets = new LinkedList<ExternalThreatBase>();
			targets.addAll(getTargetsInRange(1, pulseCannon.getRange().getMaximumDistance(), threatsInZoneRed));
			targets.addAll(getTargetsInRange(1, pulseCannon.getRange().getMaximumDistance(), threatsInZoneWhite));
			targets.addAll(getTargetsInRange(1, pulseCannon.getRange().getMaximumDistance(), threatsInZoneBlue));
			for (ExternalThreatBase target: targets) {
				target.damageBy(pulseCannon);
			}
			if (targets.size() == 0) {
				Log.log(pulseCannon.getName() + " misses.");
			}
			ResolutionThread.getInstance().doSleep();
		}
	}
	
	protected List<ExternalThreatBase> getTargetsInRange(int minimumDistance, int maximumDistance, List<ExternalThreatBase> zone) {
		List<ExternalThreatBase> found = new LinkedList<ExternalThreatBase>();
		for (ExternalThreatBase threat: zone) {
			if (threat.getDistance() >= minimumDistance) {
				if (threat.getDistance() > maximumDistance) {
					break; //no need to search further
				} else {
					if (threat.isTargetable()) {
						found.add(threat);
					}
				}
			}
		}
		return found;
	}

	protected void damageByLaser() {
		fireLaser(ship.getRedHeavyLaser(), ThreatZones.RED);
		fireLaser(ship.getRedLightLaser(), ThreatZones.RED);
		fireLaser(ship.getWhiteHeavyLaser(), ThreatZones.WHITE);
		fireLaser(ship.getBlueHeavyLaser(), ThreatZones.BLUE);
		fireLaser(ship.getBlueLightLaser(), ThreatZones.BLUE);
	}
	
	protected void fireLaser(WeaponBase laser, ThreatZones zone) {
		if (laser.hasFired()
		&& ship.getOptics().canTargetIn(zone)) {
			List<ExternalThreatBase> threatsInZone = null;
			switch (zone) {
			case BLUE:
				threatsInZone = threatsInZoneBlue; 
				break;
			case RED:
				threatsInZone = threatsInZoneRed;
				break;
			case WHITE:
				threatsInZone = threatsInZoneWhite;
				break;
			case INTERNAL:
			default:
				break;
			}
			try {
				ExternalThreatBase threat;
				if (laser instanceof WeaponRequiringReactorBase) {
					threat = getNearestHeavyLaserTargetIn(threatsInZone);
				} else {
					threat = getNearestVisibleThreat(threatsInZone);
				}
				if (threat != null) {
					threat.damageBy(laser);
				} else {
					Log.log(laser.getName() + " misses.");
				}
			} catch (IndexOutOfBoundsException e) {
				Log.log(laser.getName() + " misses.");
			} // no target in range
			ResolutionThread.getInstance().doSleep();
		}
	}

	protected void damageByRocketThrower() {
		RocketThrower rocketThrower = ship.getRocketThrower();
		if (rocketThrower.hasFired()) {
			SE2_02 juggernaut = findJuggernaut();
			if (juggernaut != null) {
				juggernaut.damageBy(rocketThrower);
			} else {
				// find targets in zone blue
				List<ExternalThreatBase> threats = new LinkedList<ExternalThreatBase>();
				ExternalThreatBase threat = getNearestRocketTargetIn(threatsInZoneBlue);
				if (threat != null) {
					threats.add(threat);
				}
				// find targets in zone white
				threat = getNearestRocketTargetIn(threatsInZoneWhite);
				if (threat != null) {
					threats.add(threat);
				}
				// find targets in zone red
				threat = getNearestRocketTargetIn(threatsInZoneRed);
				if (threat != null) {
					threats.add(threat);
				}
				
				Collections.sort(threats, threatCompareDistance);
				ExternalThreatBase target = getNearestVisibleThreat(threats);
				if (target != null) {
					if (target.getDistance() <= rocketThrower.getRange().getMaximumDistance()) {
						target.damageBy(rocketThrower);
					}
				} else {
					Log.log("Rocket finds no target.");
				}
			}
			ResolutionThread.getInstance().doSleep();
		}
	}
	
	private SE2_02 findJuggernaut() {
		for (ThreatBase threat: threats) {
			if (threat instanceof SE2_02) {
				return (SE2_02) threat;
			}
		}
		return null;
	}
	
	private ExternalThreatBase getNearestVisibleThreat(List<ExternalThreatBase> threats) {
		return getNearestVisibleThreat(threats, TargetSpecials.NOTHING);
	}
	private ExternalThreatBase getNearestVisibleThreat(List<ExternalThreatBase> threats, TargetSpecials special) {
		Iterator<ExternalThreatBase> iter = threats.iterator();
		ExternalThreatBase threat = null;
		while (iter.hasNext()) {
			threat = iter.next();
			if (threat.isTargetable()) {
				switch (special) {
				case HEAVY_LASER:
					if (threat.isTargetableHeavyLaser()) {
						return threat;
					}
					break;
				case ROCKET:
					if (threat.isTargetableByRockets()) {
						return threat;
					}
					break;
				case NOTHING:
				default:
					return threat;
				}
			};
			threat = null;
		}
		return threat;
	}

	private ExternalThreatBase getNearestHeavyLaserTargetIn(List<ExternalThreatBase> threats) {
		return getNearestVisibleThreat(threats, TargetSpecials.HEAVY_LASER);
	}

	private ExternalThreatBase getNearestRocketTargetIn(List<ExternalThreatBase> threats) {
		return getNearestVisibleThreat(threats, TargetSpecials.ROCKET);
	}
	
	/**
	 * 
	 * @return true if ship has been destroyed
	 */
	public boolean threatAction() {
		int count = threats.size();

		for (int i = 0; i < count; i++) {
			ThreatBase threat = threats.get(i);

			if (threat.getETA() <= currentTime) {	
				if (!threat.isPassedBy()
				&& !threat.isDestroyed()) {
					threat.move();
					ResolutionThread.getInstance().doSleep();
				}
				if (ship.isDestroyed()) {
					ship.destroy(threat.getName());
					return true;
				}
			}
		}
		
		ship.getRocketThrower().moveRockets();
		
		//remove passed by threats
		for (Iterator<ExternalThreatBase> iter = threatsInZoneRed.iterator(); iter.hasNext(); ) {
			ExternalThreatBase threat = iter.next();
			if (threat.isPassedBy()
			|| threat.isDestroyed()) {
				iter.remove();
			}
		}
		for (Iterator<ExternalThreatBase> iter = threatsInZoneWhite.iterator(); iter.hasNext(); ) {
			ExternalThreatBase threat = iter.next();
			if (threat.isPassedBy()
			|| threat.isDestroyed()) {
				iter.remove();
			}
		}
		for (Iterator<ExternalThreatBase> iter = threatsInZoneBlue.iterator(); iter.hasNext(); ) {
			ExternalThreatBase threat = iter.next();
			if (threat.isPassedBy()
			|| threat.isDestroyed()) {
				iter.remove();
			}
		}	
		
		Collections.sort(threatsInZoneRed, threatCompareDistance);
		Collections.sort(threatsInZoneWhite, threatCompareDistance);
		Collections.sort(threatsInZoneBlue, threatCompareDistance);
		
		return false;
	}
	
	public Space(Map map) {
		super(map);
		
		ship = new Ship(this);
		instance = this;
	}
}
