package threats;

import ship.Ship;
import spaceAlert.Log;
import spaceAlert.Nameable;
import spaceAlert.ResetableBase;
import spaceAlert.ResolutionThread;
import trajectoryBoards.TrajectoryBoardBase;

import VASSAL.command.Command;
import VASSAL.command.RemovePiece;
import VASSAL.counters.GamePiece;

public abstract class ThreatBase extends ResetableBase implements Nameable {
	protected GamePiece piece;
	protected Ship ship;
	protected TrajectoryBoardBase trajectoryBoard;
	public ThreatZones getZone() {
		return trajectoryBoard.getZone();
	}

	protected int ETA;
	public int getETA() {
		return ETA;
	}
	
	protected GamePiece ETAmarker = null;
	protected void createETAmarker() {
		ETAmarker = trajectoryBoard.getMarker(ETA);
		piece.getMap().addPiece(ETAmarker);
		trajectoryBoard.placeMarkerAt(ETAmarker, distance);
	}
	protected boolean appeared = false;
	public boolean isAppeared() {
		return appeared;
	}
	public void appear() {
		if (!isAppeared()) {
			//Log.log is moved in the sup classes
			createETAmarker();
			appeared = true;
		}
	}
	
	protected boolean targetable = true;
	public boolean isTargetable() {
		return appeared 
			&& targetable
			&& !isPhasing()
			&& !passedBy;
	}
	
	private boolean destroyed = false;
	public boolean isDestroyed() {
		return destroyed;
	}
	protected void destroy() {
		Log.log(getName() + " is destroyed.");
		destroyed = true;
		deleteETAmarker();
	}
	protected void deleteETAmarker() {
		if (ETAmarker != null) {
			Command c = new RemovePiece(ETAmarker);
			c.execute();
			ETAmarker = null;
		}
	}
	
	protected boolean passedBy = false;
	protected void passeBy() {
		passedBy = true;
		piece.setProperty("evaded", 1);
	}
	public boolean isPassedBy() {
		return passedBy;
	}
	
	protected int distance;
	public int getDistance() {
		if (appeared) {
			return distance;
		} else {
			return distance + 1;
		}
	}	
	
	private int health;
	protected int getHealth() {
		return health;
	}
	protected void setHealth(int health) {
		this.health = health;
		piece.setProperty("health", String.valueOf(health));
		if ((health < 1)
		&& (!isDestroyed())) {
			destroy();
		}
	}
	protected void increaseHealth(int increaseBy, int maxHealth) {
		if ((maxHealth - increaseBy) > health) {
			setHealth(getHealth() + increaseBy);
		} else {
			setHealth(maxHealth);
		}
	}
	protected void decreaseHealth(int decreaseBy) {
		setHealth(getHealth() - decreaseBy);
	}
	private int speed;
	protected int getSpeed() {
		return speed;
	}
	protected void setSpeed(int speed) {
		this.speed = speed;
		piece.setProperty("speed", String.valueOf(speed));
	}
	protected void increaseSpeed(int increaseBy) {
		setSpeed(getSpeed() + increaseBy);
	}
	
	private boolean phasing = false;
	public boolean isPhasing() {
		return phasing;
	}
	protected void setPhasing(boolean phasing) {
		this.phasing = phasing;
	}
	
	
	protected abstract void doXAction();
	protected abstract void doYAction();
	protected abstract void doZAction();
		
	public void move() {
		if (isAppeared() 
		&& !isDestroyed()
		&& (distance > 0)) {
			move(speed);
		}
	}
	
	public void move(int speed) {
		if (isAppeared()
		&& !isDestroyed()
		&& (distance > 0)) {
			Log.log(getName() + " advances.");
			Move:
			for (int i = distance - 1; i >= distance - speed; i--) {
				if (i > 0) {
					trajectoryBoard.placeMarkerAt(ETAmarker, i);
				}
				switch (trajectoryBoard.getActionAt(i)) {
				case X:
					doXAction();
					ResolutionThread.getInstance().doSleep();
					break;
				case Y:
					doYAction();
					ResolutionThread.getInstance().doSleep();
					break;
				case Z:
					doZAction();
					passeBy();
					ResolutionThread.getInstance().doSleep();
					break Move;
				case NOTHING:
					break;
				}
			}
			distance -= speed;
			
			if (!isPassedBy()) {
				trajectoryBoard.placeMarkerAt(ETAmarker, distance);
			} else {
				Log.log(getName() + " passes by.");
				deleteETAmarker();
			}
		}
	}
	
	public String getName() {
		return (String) piece.getProperty("name");
	}
	
	protected int scoreSurvived;
	protected int scoreDestroyed;
	public int getScore() {
		if (isDestroyed()) {
			return scoreDestroyed;
		} else {
			return scoreSurvived;
		}
	}
	
	public ThreatBase(Ship ship, GamePiece piece, TrajectoryBoardBase trajectoryBoard) {
		super(ship);
		this.ship = ship;
		
		this.piece = piece;
		piece.setProperty("evaded", 0);
		
		this.trajectoryBoard = trajectoryBoard;
		distance = trajectoryBoard.getMaxDistance();
		
		try {
			ETA = Integer.valueOf((String)piece.getProperty("ETA"));
		} catch (NumberFormatException e) {
			ETA = Integer.MAX_VALUE;
		}
	}
}
