package threats;

import ship.Ship;
import spaceAlert.Log;
import trajectoryBoards.TrajectoryBoardBase;
import whiteThreats.*;
import yellowThreats.*;
import VASSAL.counters.GamePiece;

public class ThreatGenerator {
	public static TrajectoryBoardBase[] trajectoryBoards = new TrajectoryBoardBase[ThreatZones.values().length + 1];
	
	public static ThreatBase createThreat(Ship ship, GamePiece piece) {
		String location = piece.getMap().locationName(piece.getPosition());
		
		TrajectoryBoardBase trajectoryBoard = trajectoryBoards[ThreatZones.RED.ordinal()];
		if (location.startsWith("Threat: White")) {
			trajectoryBoard = trajectoryBoards[ThreatZones.WHITE.ordinal()];
		} else if (location.startsWith("Threat: Blue")) {
			trajectoryBoard = trajectoryBoards[ThreatZones.BLUE.ordinal()];
		} else if (location.startsWith("Threat: Internal")) {
			trajectoryBoard = trajectoryBoards[ThreatZones.INTERNAL.ordinal()];
		}
		String name = (String) piece.getProperty("intern");
		ThreatBase threat = createWhiteThreat(ship, piece, trajectoryBoard, name);
		if (threat == null) {
			threat = createYellowThreat(ship, piece, trajectoryBoard, name);
		}
		
		if (threat == null) {
			Log.error("Couldn't found threat " + name);
			threat = new ThreatDummy(ship, piece, trajectoryBoard);
		}
		return threat;
	}
	
	private static ThreatBase createYellowThreat(Ship ship,
			GamePiece piece, TrajectoryBoardBase trajectoryBoard, String name) {
		ThreatBase threat = createYellowExternalThreat(ship, piece, trajectoryBoard, name);
		if (threat == null) {
			threat = createYellowInternalThreat(ship, piece, trajectoryBoard, name);
		}
		if (threat == null) {
			threat = createYellowSeriousThreat(ship, piece, trajectoryBoard, name);
		}
		return threat;
	}
	
	private static ExternalThreatBase createYellowExternalThreat(
			Ship ship, GamePiece piece,
			TrajectoryBoardBase trajectoryBoard, String name) {
		if (name.equals("E2-01")) {
			return new E2_01(ship, piece, trajectoryBoard);
		} else if (name.equals("E2-02")) {
			return new E2_02(ship, piece, trajectoryBoard);
		} else if (name.equals("E2-03")) {
			return new E2_03(ship, piece, trajectoryBoard);
		} else if (name.equals("E2-04")) {
			return new E2_04(ship, piece, trajectoryBoard);
		} else if (name.equals("E2-05")) {
			return new E2_05(ship, piece, trajectoryBoard);
		} else if (name.equals("E2-06")) {
			return new E2_06(ship, piece, trajectoryBoard);
		} else if (name.equals("E2-07")) {
			return new E2_07(ship, piece, trajectoryBoard);
		} else if (name.equals("E2-101")) {
			return new E2_101(ship, piece, trajectoryBoard);
		} else if (name.equals("E2-102")) {
			return new E2_102(ship, piece, trajectoryBoard);
		} else if (name.equals("E2-103")) {
			return new E2_103(ship, piece, trajectoryBoard);
		} else {
			return null;
		}
	}
	
	private static InternalThreatBase createYellowInternalThreat(
			Ship ship, GamePiece piece,
			TrajectoryBoardBase trajectoryBoard, String name) {
		if (name.equals("I2-01")) {
			return new I2_01(ship, piece, trajectoryBoard);
		} else if (name.equals("I2-02")) {
			return new I2_02(ship, piece, trajectoryBoard);
		} else if (name.equals("I2-03")) {
			return new I2_03(ship, piece, trajectoryBoard);
		} else if (name.equals("I2-04")) {
			return new I2_04(ship, piece, trajectoryBoard);
		} else if (name.equals("I2-05")) {
			return new I2_05(ship, piece, trajectoryBoard);
		} else if (name.equals("I2-06")) {
			return new I2_06(ship, piece, trajectoryBoard);
		} else if (name.equals("I2-101")) {
			return new I2_101(ship, piece, trajectoryBoard);
		} else if (name.equals("I2-102")) {
			return new I2_102(ship, piece, trajectoryBoard);
		} else {
			return null;
		}
	}
	
	private static ThreatBase createYellowSeriousThreat(Ship ship,
			GamePiece piece, TrajectoryBoardBase trajectoryBoard, String name) {
		ThreatBase threat = createYellowSeriousExternalThreat(ship, piece, trajectoryBoard, name);
		if (threat == null) {
			threat = createYellowSeriousInternalThreat(ship, piece, trajectoryBoard, name);
		}
		return threat;
	}
	
	private static ExternalThreatBase createYellowSeriousExternalThreat(Ship ship,
			GamePiece piece, TrajectoryBoardBase trajectoryBoard, String name) {
		if (name.equals("SE2-01")) {
			return new SE2_01(ship, piece, trajectoryBoard);
		} else if (name.equals("SE2-02")) {
			return new SE2_02(ship, piece, trajectoryBoard);
		} else if (name.equals("SE2-03")) {
			return new SE2_03(ship, piece, trajectoryBoard);
		} else if (name.equals("SE2-04")) {
			return new SE2_04(ship, piece, trajectoryBoard);
		} else if (name.equals("SE2-05")) {
			return new SE2_05(ship, piece, trajectoryBoard);
		} else if (name.equals("SE2-06")) {
			return new SE2_06(ship, piece, trajectoryBoard);
		} else if (name.equals("SE2-101")) {
			return new SE2_101(ship, piece, trajectoryBoard);
		} else if (name.equals("SE2-102")) {
			return new SE2_102(ship, piece, trajectoryBoard);
		} else if (name.equals("SE2-103")) {
			return new SE2_103(ship, piece, trajectoryBoard);
		} else {
			return null;
		}
	}
	
	private static InternalThreatBase createYellowSeriousInternalThreat(Ship ship,
			GamePiece piece, TrajectoryBoardBase trajectoryBoard, String name) {
		if (name.equals("SI2-01")) {
			return new SI2_01(ship, piece, trajectoryBoard);
		} else if (name.equals("SI2-02")) {
			return new SI2_02(ship, piece, trajectoryBoard);
		} else if (name.equals("SI2-03")) {
			return new SI2_03(ship, piece, trajectoryBoard);
		} else if (name.equals("SI2-04")) {
			return new SI2_04(ship, piece, trajectoryBoard);
		} else if (name.equals("SI2-05")) {
			return new SI2_05(ship, piece, trajectoryBoard);
		} else if (name.equals("SI2-101")) {
			return new SI2_101(ship, piece, trajectoryBoard);
		} else if (name.equals("SI2-102")) {
			return new SI2_102(ship, piece, trajectoryBoard);
		} else {
			return null;
		}
	}
	
	private static ThreatBase createWhiteThreat(Ship ship,
			GamePiece piece, TrajectoryBoardBase trajectoryBoard, String name) {
		ThreatBase threat = createWhiteExternalThreat(ship, piece, trajectoryBoard, name);
		if (threat == null) {
			threat = createWhiteInternalThreat(ship, piece, trajectoryBoard, name);
		}
		if (threat == null) {
			threat = createWhiteSeriousThreat(ship, piece, trajectoryBoard, name);
		}
		return threat;
	}

	private static ExternalThreatBase createWhiteExternalThreat(
			Ship ship, GamePiece piece,
			TrajectoryBoardBase trajectoryBoard, String name) {
		if (name.equals("E1-01")) {
			return new E1_01(ship, piece, trajectoryBoard);
		} else if (name.equals("E1-02")) {
			return new E1_02(ship, piece, trajectoryBoard);
		} else if (name.equals("E1-03")) {
			return new E1_03(ship, piece, trajectoryBoard);
		} else if (name.equals("E1-04")) {
			return new E1_04(ship, piece, trajectoryBoard);
		} else if (name.equals("E1-05")) {
			return new E1_05(ship, piece, trajectoryBoard);
		} else if (name.equals("E1-06")) {
			return new E1_06(ship, piece, trajectoryBoard);
		} else if (name.equals("E1-07")) {
			return new E1_07(ship, piece, trajectoryBoard);
		} else if (name.equals("E1-08")) {
			return new E1_08(ship, piece, trajectoryBoard);
		} else if (name.equals("E1-09")) {
			return new E1_09(ship, piece, trajectoryBoard);
		} else if (name.equals("E1-10")) {
			return new E1_10(ship, piece, trajectoryBoard);
		} else if (name.equals("E1-101")) {
			return new E1_101(ship, piece, trajectoryBoard);
		} else if (name.equals("E1-102")) {
			return new E1_102(ship, piece, trajectoryBoard);
		} else {
			return null;
		}
	}
	
	private static InternalThreatBase createWhiteInternalThreat(
			Ship ship, GamePiece piece,
			TrajectoryBoardBase trajectoryBoard, String name) {
		if (name.equals("I1-01")) {
			return new I1_01(ship, piece, trajectoryBoard);
		} else if (name.equals("I1-02")) {
			return new I1_02(ship, piece, trajectoryBoard);
		} else if (name.equals("I1-03")) {
			return new I1_03(ship, piece, trajectoryBoard);
		} else if (name.equals("I1-04")) {
			return new I1_04(ship, piece, trajectoryBoard);
		} else if (name.equals("I1-05")) {
			return new I1_05(ship, piece, trajectoryBoard);
		} else if (name.equals("I1-06")) {
			return new I1_06(ship, piece, trajectoryBoard);
		} else if (name.equals("I1-07")) {
			return new I1_07(ship, piece, trajectoryBoard);
		} else if (name.equals("I1-101")) {
			return new I1_101(ship, piece, trajectoryBoard);
		} else {
			return null;
		}
	}
	
	private static ThreatBase createWhiteSeriousThreat(Ship ship,
			GamePiece piece, TrajectoryBoardBase trajectoryBoard, String name) {
		ThreatBase threat = createWhiteSeriousExternalThreat(ship, piece, trajectoryBoard, name);
		if (threat == null) {
			threat = createWhiteSeriousInternalThreat(ship, piece, trajectoryBoard, name);
		}
		return threat;
	}

	private static ExternalThreatBase createWhiteSeriousExternalThreat(Ship ship,
			GamePiece piece, TrajectoryBoardBase trajectoryBoard, String name) {
		if (name.equals("SE1-01")) {
			return new SE1_01(ship, piece, trajectoryBoard);
		} else if (name.equals("SE1-02")) {
			return new SE1_02(ship, piece, trajectoryBoard);
		} else if (name.equals("SE1-03")) {
			return new SE1_03(ship, piece, trajectoryBoard);
		} else if (name.equals("SE1-04")) {
			return new SE1_04(ship, piece, trajectoryBoard);
		} else if (name.equals("SE1-05")) {
			return new SE1_05(ship, piece, trajectoryBoard);
		} else if (name.equals("SE1-06")) {
			return new SE1_06(ship, piece, trajectoryBoard);
		} else if (name.equals("SE1-07")) {
			return new SE1_07(ship, piece, trajectoryBoard);
		} else if (name.equals("SE1-08")) {
			return new SE1_08(ship, piece, trajectoryBoard);
		} else if (name.equals("SE1-101")) {
			return new SE1_101(ship, piece, trajectoryBoard);
		} else if (name.equals("SE1-102")) {
			return new SE1_102(ship, piece, trajectoryBoard);
		} else {
			return null;
		}
	}
	
	private static InternalThreatBase createWhiteSeriousInternalThreat(Ship ship,
			GamePiece piece, TrajectoryBoardBase trajectoryBoard, String name) {
		if (name.equals("SI1-01")) {
			return new SI1_01(ship, piece, trajectoryBoard);
		} else if (name.equals("SI1-02")) {
			return new SI1_02(ship, piece, trajectoryBoard);
		} else if (name.equals("SI1-03")) {
			return new SI1_03(ship, piece, trajectoryBoard);
		} else if (name.equals("SI1-04")) {
			return new SI1_04(ship, piece, trajectoryBoard);
		} else if (name.equals("SI1-05")) {
			return new SI1_05(ship, piece, trajectoryBoard);
		} else if (name.equals("SI1-06")) {
			return new SI1_06(ship, piece, trajectoryBoard);
		} else if (name.equals("SI1-101")) {
			return new SI1_101(ship, piece, trajectoryBoard);
		} else {
			return null;
		}
	}
}
