package whiteThreats;

import VASSAL.counters.GamePiece;
import ship.Player;
import ship.Ship;
import ship.ShipComponentBase;
import ship.ShipZones;
import spaceAlert.Log;
import spaceAlert.Nameable;
import threats.InternalMultipleAttachedThreatBase;
import trajectoryBoards.TrajectoryBoardBase;

public class SI1_06 extends InternalMultipleAttachedThreatBase {
	//Battlebotaufstand
	public SI1_06(Ship ship, GamePiece piece,
			TrajectoryBoardBase trajectoryBoard) {
		super(ship, piece, trajectoryBoard);

		scoreSurvived = 4;
		scoreDestroyed= 8;
		
		setSpeed(2);
		setHealth(4);
		
		action = "C";
		location = ShipZones.RED_LOWER;
		additionalLocation.add(ShipZones.BLUE_UPPER);
		attachedTo = ship.getRedBattleBots();
		additionalAttachedTo.add(ship.getBlueBattleBots());
	}
	
	private ShipComponentBase damagedAtStation = null;
	
	@Override
	public void damage(Nameable player, ShipComponentBase station, boolean isHeroic) {
		super.damage(player, station, isHeroic);
		
		if (damagedAtStation == null) {
			damagedAtStation = station;
		} else if (damagedAtStation != station) {
			setHealth(getHealth() - 1);
			damagedAtStation = null;
		}
	}
	
	@Override
	public void reset() {
		super.reset();
		
		damagedAtStation = null;
	}

	@Override
	protected void doXAction() {
		for (Player player: ship.getPlayers()) {
			if (player.hasBattlebots()) {
				Log.log(getName() + " knocks out " + player.getName() + ".");
				player.knockOut();
			}
		}
	}

	@Override
	protected void doYAction() {
		Log.log(getName() + " knocks out all players in " + location.getName() + ".");
		ship.knockOutPlayersIn(location);
		for (ShipZones location: additionalLocation) {
			Log.log(getName() + " knocks out all players in " + location.getName() + ".");
			ship.knockOutPlayersIn(location);
		}
	}

	@Override
	protected void doZAction() {
		Log.log(getName() + " knocks out all players which aren't at " + ShipZones.WHITE_UPPER.getName() + ".");
		for (Player player: ship.getPlayers()) {
			if (player.getLocation() != ShipZones.WHITE_UPPER) {
				player.knockOut();
			}
		}
	}

}
