package yellowThreats;

import java.util.LinkedList;
import java.util.List;

import VASSAL.counters.GamePiece;
import ship.Ship;
import ship.ShipZones;
import spaceAlert.GamePieceCreator;
import spaceAlert.Log;
import spaceAlert.Space;
import threats.InternalMovingThreatBase;
import threats.ThreatBase;
import trajectoryBoards.TrajectoryBoardBase;

public class I2_01 extends InternalMovingThreatBase {
	//Schleim
	List<I2_01_02_Progeny> progenies = new LinkedList<I2_01_02_Progeny>();

	public I2_01(Ship ship, GamePiece piece, TrajectoryBoardBase trajectoryBoard) {
		super(ship, piece, trajectoryBoard);

		scoreSurvived = 3;
		scoreDestroyed= 6;
		
		setSpeed(2);
		setHealth(2);
		fightsBack = false;
		tokenName = "Token I2_01|02";
		location = ShipZones.BLUE_LOWER;
		
		staysAfterPassingBy = true;
	}
	
	@Override
	public boolean isDestroyed() {
		boolean destroyed = super.isDestroyed();
		if (destroyed) {
			for (I2_01_02_Progeny progeny: progenies) {
				if (!progeny.isDestroyed()) {
					return false;
				}
			}
		}
		return destroyed;
	}
	
	@Override
	protected void destroy() {
		if (isDestroyed()) {
			super.destroy();
		} else {
			if (getHealth() < 1) {
				Log.log(getName() + " is destroyed");
				deleteToken();
			}
		}
	}
	
	@Override
	protected void doXAction() {
		Log.log(getName() + " destroys one rocket.");
		ship.getRocketThrower().deleteRocket(this);
	}
	
	protected ShipZones getProgenyLocation(ShipZones parentLocation) {
		switch (parentLocation) {
		case BLUE_LOWER:
			return ShipZones.WHITE_LOWER;
		case WHITE_LOWER:
			return ShipZones.RED_LOWER;
		default:
			return null;		
		}
	}
	
	private I2_01_02_Progeny createProgeny(I2_01 parent) {
		if (parent.isDestroyed()) {
			return null;
		}
		ShipZones location = getProgenyLocation(parent.getLocation());
		if (location == null) {
			return null;
		} else {
			List<ThreatBase> threats = Space.getInstance().getThreats();
			for (ThreatBase threat: threats) {
				if (threat instanceof I2_01) {
					if (((I2_01) threat).getLocation() == location) {
						return null;
					}
				}
			}
			GamePiece piece = GamePieceCreator.getInstance().get("hidden");
			I2_01_02_Progeny progeny = new I2_01_02_Progeny(ship, piece, trajectoryBoard, this, location);
			threats.add(progeny);
			
			return progeny;
		}
	}

	@Override
	protected void doYAction() {
		Log.log(getName() + " spreads.");
		int count = progenies.size();
		for (int i = 0; i < count; i++) {
			I2_01 progeny = progenies.get(i);
			if (!progeny.isDestroyed()) {
				I2_01_02_Progeny grandProgeny = createProgeny(progeny);
				if (grandProgeny != null) {
					progenies.add(grandProgeny);
				}
			}
		}

		if (getHealth() > 0) {
			I2_01_02_Progeny progeny = createProgeny(this);
			if (progeny != null) {
				progenies.add(progeny);
			}
		}
	}

	@Override
	protected void doZAction() {
		if (getHealth() > 0) {
			Log.log(getName() + " attacks 2.");
			ship.damage(getZone(), 2);
		}
		
		for (I2_01_02_Progeny progeny: progenies) {
			if (!progeny.isDestroyed()) {
				Log.log("Progeny " + getName() + " attacks 2.");
				ship.damage(progeny.getZone(), 2);
			}
		}
	}

}
