package yellowThreats;

import java.util.List;

import VASSAL.counters.GamePiece;
import ship.Ship;
import ship.ShipZones;
import spaceAlert.GamePieceCreator;
import spaceAlert.Log;
import spaceAlert.Space;
import threats.InternalMovingThreatBase;
import threats.ThreatBase;
import trajectoryBoards.TrajectoryBoardBase;

public class SI2_04 extends InternalMovingThreatBase {
	//Kontaminierung
	public SI2_04(Ship ship, GamePiece piece,
			TrajectoryBoardBase trajectoryBoard) {
		super(ship, piece, trajectoryBoard);

		scoreSurvived = 6;
		scoreDestroyed= 12;
		
		setSpeed(2);
		setHealth(3);
		
		tokenName = "Token SI2_04";
		location = ShipZones.FIGHTERS;
		
		GamePiece childPiece = GamePieceCreator.getInstance().get("hidden");		
		childs[0] = new SI2_04_child(this, ship, childPiece, trajectoryBoard, ShipZones.RED_UPPER);
		childs[1] = new SI2_04_child(this, ship, childPiece, trajectoryBoard, ShipZones.RED_LOWER);
		childs[2] = new SI2_04_child(this, ship, childPiece, trajectoryBoard, ShipZones.BLUE_UPPER);
		childs[3] = new SI2_04_child(this, ship, childPiece, trajectoryBoard, ShipZones.BLUE_LOWER);
		List<ThreatBase> threats = Space.getInstance().getThreats();
		for (SI2_04_child child: childs) {
			threats.add(child);
		}
	}
	
	private SI2_04_child[] childs = new SI2_04_child[4];
	
	@Override
	public void appear() {
		if (!isAppeared()) {
			createETAmarker();
			appeared = true;
			
			for (SI2_04_child child: childs) {
				child.appear();
			}
		}
	}
	
	@Override
	protected void destroy() {
		super.destroy();
		
		for (SI2_04_child child: childs) {
			if (!child.isDestroyed()) {
				child.destroy();
			}
		}
	}
	
	@Override
	public void createToken() {}
	
	@Override
	public void moveTo(ShipZones location) {}
		
	@Override
	protected void doXAction() {
		for (SI2_04_child child: childs) {
			if (!child.isDestroyed()) {
				Log.log(getName() + " delays players in " + child.getLocation().getName() + ".");
				ship.delayPlayersIn(child.getLocation());
			}
		}
	}

	@Override
	protected void doYAction() {
		for (SI2_04_child child: childs) {
			if (!child.isDestroyed()) {
				Log.log(getName() + " attacks 1.");
				ship.damage(child.getZone(), 1);
			}
		}
	}

	@Override
	protected void doZAction() {
		for (SI2_04_child child: childs) {
			if (!child.isDestroyed()) {
				Log.log(getName() + " knocks out players in " + child.getLocation().getName() + ".");
				ship.knockOutPlayersIn(child.getLocation());
			}
		}
	}
}
